@@ -1,12 +1,16 @@
 {-# LANGUAGE ConstraintKinds #-}
 {-# LANGUAGE DataKinds #-}
 {-# LANGUAGE ExplicitNamespaces #-}
+{-# LANGUAGE FlexibleInstances #-}
 {-# LANGUAGE GADTs #-}
 {-# LANGUAGE MagicHash #-}
 {-# LANGUAGE PatternSynonyms #-}
 {-# LANGUAGE PolyKinds #-}
+{-# LANGUAGE QuantifiedConstraints #-}
+{-# LANGUAGE ScopedTypeVariables #-}
 {-# LANGUAGE StandaloneDeriving #-}
 {-# LANGUAGE TemplateHaskell #-}
+{-# LANGUAGE TypeApplications #-}
 {-# LANGUAGE TypeFamilies #-}
 {-# LANGUAGE TypeOperators #-}
 {-# LANGUAGE UndecidableInstances #-}
@@ -35,7 +39,7 @@
 module Data.Singletons (
   -- * Main singleton definitions
 
-  Sing(SLambda, applySing), (@@),
+  Sing, SLambda(..), (@@),
 
   SingI(..), SingKind(..),
 
@@ -49,6 +53,10 @@ module Data.Singletons (
   singByProxy#,
   withSing, singThat,
 
+  -- ** @WrappedSing@
+  WrappedSing(..), SWrappedSing(..), UnwrapSing,
+  -- $SingletonsOfSingletons
+
   -- ** Defunctionalization
   TyFun, type (~>),
   TyCon1, TyCon2, TyCon3, TyCon4, TyCon5, TyCon6, TyCon7, TyCon8,
@@ -85,6 +93,7 @@ module Data.Singletons (
 import Data.Singletons.Internal
 import Data.Singletons.Prelude.Enum
 import Data.Singletons.Prelude.Eq
+import Data.Singletons.Prelude.Instances
 import Data.Singletons.Prelude.IsString
 import Data.Singletons.Prelude.Monoid
 import Data.Singletons.Prelude.Num
@@ -92,9 +101,23 @@ import Data.Singletons.Prelude.Ord
 import Data.Singletons.Prelude.Semigroup
 import Data.Singletons.Promote
 import Data.Singletons.ShowSing
+import Data.Singletons.Single (singITyConInstances)
 import Data.String
 import qualified Data.Text as T (pack)
 
+----------------------------------------------------------------------
+---- SingI TyCon{N} instances ----------------------------------------
+----------------------------------------------------------------------
+
+{-
+Generates SingI instances for TyCon1 through TyCon8:
+
+  instance (forall a.    SingI a           => SingI (f a),   ...) => SingI (TyCon1 f)
+  instance (forall a b. (SingI a, SingI b) => SingI (f a b), ...) => SingI (TyCon2 f)
+  ...
+-}
+$(singITyConInstances [1..8])
+
 ----------------------------------------------------------------------
 ---- SomeSing instances ----------------------------------------------
 ----------------------------------------------------------------------
@@ -114,15 +137,21 @@ instance SBounded k => Bounded (SomeSing k) where
   minBound = SomeSing sMinBound
   maxBound = SomeSing sMaxBound
 
-instance (SEnum k, SingKind k) => Enum (SomeSing k) where
+instance SEnum k => Enum (SomeSing k) where
   succ (SomeSing a) = SomeSing (sSucc a)
   pred (SomeSing a) = SomeSing (sPred a)
   toEnum n = withSomeSing (fromIntegral n) (SomeSing . sToEnum)
   fromEnum (SomeSing a) = fromIntegral (fromSing (sFromEnum a))
   enumFromTo (SomeSing from) (SomeSing to) =
-    map toSing (fromSing (sEnumFromTo from to))
+    listFromSingShallow (sEnumFromTo from to)
   enumFromThenTo (SomeSing from) (SomeSing then_) (SomeSing to) =
-    map toSing (fromSing (sEnumFromThenTo from then_ to))
+    listFromSingShallow (sEnumFromThenTo from then_ to)
+
+-- Like the 'fromSing' implementation for lists, but bottoms out at
+-- 'SomeSing' instead of recursively invoking 'fromSing'.
+listFromSingShallow :: SList (x :: [a]) -> [SomeSing a]
+listFromSingShallow SNil         = []
+listFromSingShallow (SCons x xs) = SomeSing x : listFromSingShallow xs
 
 instance SNum k => Num (SomeSing k) where
   SomeSing a + SomeSing b = SomeSing (a %+ b)
@@ -133,7 +162,10 @@ instance SNum k => Num (SomeSing k) where
   signum (SomeSing a) = SomeSing (sSignum a)
   fromInteger n = withSomeSing (fromIntegral n) (SomeSing . sFromInteger)
 
-deriving instance ShowSing k => Show (SomeSing k)
+instance ShowSing k => Show (SomeSing k) where
+  showsPrec p (SomeSing (s :: Sing a)) =
+    showParen (p > 10) $ showString "SomeSing " . showsPrec 11 s
+      :: ShowSing' a => ShowS
 
 instance SSemigroup k => Semigroup (SomeSing k) where
   SomeSing a <> SomeSing b = SomeSing (a %<> b)
@@ -149,7 +181,76 @@ instance SIsString k => IsString (SomeSing k) where
 ----------------------------------------------------------------------
 
 $(genDefunSymbols [''Demote, ''SameKind, ''KindOf, ''(~>), ''Apply, ''(@@)])
--- SingFunction1 et al. are not defunctionalizable at the moment due to #198
+-- WrapSing, UnwrapSing, and SingFunction1 et al. are not defunctionalizable
+-- at the moment due to Trac #9269
+
+{- $SingletonsOfSingletons
+
+Aside from being a data type to hang instances off of, 'WrappedSing' has
+another purpose as a general-purpose mechanism for allowing one to write
+code that uses singletons of other singletons. For instance, suppose you
+had the following data type:
+
+@
+data T :: Type -> Type where
+  MkT :: forall a (x :: a). 'Sing' x -> F a -> T a
+@
+
+A naïve attempt at defining a singleton for @T@ would look something like
+this:
+
+@
+data ST :: forall a. T a -> Type where
+  SMkT :: forall a (x :: a) (sx :: 'Sing' x) (f :: F a).
+          'Sing' sx -> 'Sing' f -> ST (MkT sx f)
+@
+
+But there is a problem here: what exactly /is/ @'Sing' sx@? If @x@ were 'True',
+for instance, then @sx@ would be 'STrue', but it's not clear what
+@'Sing' 'STrue'@ should be. One could define @SSBool@ to be the singleton of
+'SBool's, but in order to be thorough, one would have to generate a singleton
+for /every/ singleton type out there. Plus, it's not clear when to stop. Should
+we also generate @SSSBool@, @SSSSBool@, etc.?
+
+Instead, 'WrappedSing' and its singleton 'SWrappedSing' provide a way to talk
+about singletons of other arbitrary singletons without the need to generate a
+bazillion instances. For reference, here is the definition of 'SWrappedSing':
+
+@
+newtype 'SWrappedSing' :: forall k (a :: k). 'WrappedSing' a -> Type where
+  'SWrapSing' :: forall k (a :: k) (ws :: 'WrappedSing' a).
+                 { 'sUnwrapSing' :: 'Sing' a } -> 'SWrappedSing' ws
+type instance 'Sing' \@('WrappedSing' a) = 'SWrappedSing'
+@
+
+'SWrappedSing' is a bit of an unusual singleton in that its field is a
+singleton for @'Sing' \@k@, not @'WrappedSing' \@k@. But that's exactly the
+point—a singleton of a singleton contains as much type information as the
+underlying singleton itself, so we can get away with just @'Sing' \@k@.
+
+As an example of this in action, here is how you would define the singleton
+for the earlier @T@ type:
+
+@
+data ST :: forall a. T a -> Type where
+  SMkT :: forall a (x :: a) (sx :: 'Sing' x) (f :: F a).
+          'Sing' ('WrapSing' sx) -> 'Sing' f -> ST (MkT sx f)
+@
+
+With this technique, we won't need anything like @SSBool@ in order to
+instantiate @x@ with 'True'. Instead, the field of type
+@'Sing' ('WrapSing' sx)@ will simply be a newtype around 'SBool'. In general,
+you'll need /n/ layers of 'WrapSing' if you wish to single a singleton /n/
+times.
+
+Note that this is not the only possible way to define a singleton for @T@.
+An alternative approach that does not make use of singletons-of-singletons is
+discussed at some length
+<https://github.com/goldfirere/singletons/issues/366#issuecomment-489469086 here>.
+Due to the technical limitations of this approach, however, we do not use it
+in @singletons@ at the moment, instead favoring the
+slightly-clunkier-but-more-reliable 'WrappedSing' approach.
+-}
 
 {- $SLambdaPatternSynonyms
 
