@@ -60,10 +60,11 @@ import Data.Singletons.Prelude.Bool
 -- and its singleton. However, because @Rep@ is promoted to @*@, the singleton
 -- is perhaps slightly unexpected:
 --
--- > data instance Sing (a :: *) where
+-- > data SRep (a :: *) where
 -- >   SNat :: Sing Nat
 -- >   SBool :: Sing Bool
 -- >   SMaybe :: Sing a -> Sing (Maybe a)
+-- > type instance Sing = SRep
 --
 -- The unexpected part is that @Nat@, @Bool@, and @Maybe@ above are the real @Nat@,
 -- @Bool@, and @Maybe@, not just promoted data constructors.
@@ -76,7 +77,7 @@ singletonStar names = do
   kinds <- mapM getKind names
   ctors <- zipWithM (mkCtor True) names kinds
   let repDecl = DDataD Data [] repName [] (Just (DConT typeKindName)) ctors
-                         [DDerivClause Nothing (map DConPr [''Eq, ''Ord, ''Read, ''Show])]
+                         [DDerivClause Nothing (map DConT [''Eq, ''Ord, ''Read, ''Show])]
   fakeCtors <- zipWithM (mkCtor False) names kinds
   let dataDecl = DataDecl repName [] fakeCtors
   -- Why do we need withLocalDeclarations here? It's because we end up
@@ -86,10 +87,10 @@ singletonStar names = do
   withLocalDeclarations (decToTH repDecl) $ do
     -- We opt to infer the constraints for the Eq instance here so that when it's
     -- promoted, Rep will be promoted to Type.
-    dataDeclEqCxt <- inferConstraints (DConPr ''Eq) (DConT repName) fakeCtors
-    let dataDeclEqInst = DerivedDecl (Just dataDeclEqCxt) (DConT repName) dataDecl
+    dataDeclEqCxt <- inferConstraints (DConT ''Eq) (DConT repName) fakeCtors
+    let dataDeclEqInst = DerivedDecl (Just dataDeclEqCxt) (DConT repName) repName dataDecl
     ordInst  <- mkOrdInstance Nothing (DConT repName) dataDecl
-    showInst <- mkShowInstance Nothing (DConT repName) dataDecl
+    showInst <- mkShowInstance ForPromotion Nothing (DConT repName) dataDecl
     (pInsts, promDecls) <- promoteM [] $ do promoteDataDec dataDecl
                                             promoteDerivedEqDec dataDeclEqInst
                                             traverse (promoteInstanceDec mempty)
@@ -131,21 +132,24 @@ singletonStar names = do
               noBang = Bang NoSourceUnpackedness NoSourceStrictness
 
         -- demote a kind back to a type, accumulating any unbound parameters
-        kindToType :: DsMonad q => [DType] -> DKind -> QWithAux [Name] q DType
+        kindToType :: DsMonad q => [DTypeArg] -> DKind -> QWithAux [Name] q DType
         kindToType _    (DForallT _ _ _) = fail "Explicit forall encountered in kind"
         kindToType args (DAppT f a) = do
           a' <- kindToType [] a
-          kindToType (a' : args) f
+          kindToType (DTANormal a' : args) f
+        kindToType args (DAppKindT f a) = do
+          a' <- kindToType [] a
+          kindToType (DTyArg a' : args) f
         kindToType args (DSigT t k) = do
           t' <- kindToType [] t
           k' <- kindToType [] k
-          return $ DSigT t' k' `foldType` args
+          return $ DSigT t' k' `applyDType` args
         kindToType args (DVarT n) = do
           addElement n
-          return $ DVarT n `foldType` args
-        kindToType args (DConT n)    = return $ DConT name    `foldType` args
+          return $ DVarT n `applyDType` args
+        kindToType args (DConT n)    = return $ DConT name `applyDType` args
           where name | isTypeKindName n = repName
                      | otherwise        = n
-        kindToType args DArrowT      = return $ DArrowT       `foldType` args
-        kindToType args k@(DLitT {}) = return $ k             `foldType` args
-        kindToType args DWildCardT   = return $ DWildCardT    `foldType` args
+        kindToType args DArrowT      = return $ DArrowT    `applyDType` args
+        kindToType args k@(DLitT {}) = return $ k          `applyDType` args
+        kindToType args DWildCardT   = return $ DWildCardT `applyDType` args
