@@ -33,15 +33,15 @@ mkEnumInstance mb_ctxt ty (DataDecl data_name tvbs cons) = do
               non_vanilla || not (null $ tysOfConFields f)) cons) $
     fail ("Can't derive Enum instance for " ++ pprint (typeToTH ty) ++ ".")
   n <- qNewName "n"
-  let to_enum = UFunction [DClause [DVarPa n] (to_enum_rhs cons [0..])]
+  let to_enum = UFunction [DClause [DVarP n] (to_enum_rhs cons [0..])]
       to_enum_rhs [] _ = DVarE errorName `DAppE` DLitE (StringL "toEnum: bad argument")
       to_enum_rhs (DCon _ _ name _ _ : rest) (num:nums) =
         DCaseE (DVarE equalsName `DAppE` DVarE n `DAppE` DLitE (IntegerL num))
-          [ DMatch (DConPa trueName []) (DConE name)
-          , DMatch (DConPa falseName []) (to_enum_rhs rest nums) ]
+          [ DMatch (DConP trueName []) (DConE name)
+          , DMatch (DConP falseName []) (to_enum_rhs rest nums) ]
       to_enum_rhs _ _ = error "Internal error: exhausted infinite list in to_enum_rhs"
 
-      from_enum = UFunction (zipWith (\i con -> DClause [DConPa (extractName con) []]
+      from_enum = UFunction (zipWith (\i con -> DClause [DConP (extractName con) []]
                                                         (DLitE (IntegerL i)))
                                      [0..] cons)
   return (InstDecl { id_cxt     = fromMaybe [] mb_ctxt
