@@ -20,8 +20,6 @@ import Language.Haskell.TH.Syntax
 import Data.Singletons.Deriving.Util
 import Data.Singletons.Util
 import Data.List
-import Data.List.NonEmpty (NonEmpty(..))
-import Data.Generics.Twins
 
 -- @inferConstraints cls inst_ty cons@ infers the instance context for a
 -- derived type class instance of @cls@ for @inst_ty@, using the constructors
@@ -63,7 +61,7 @@ import Data.Generics.Twins
 -- For more information on what this means, refer to the documentation for
 -- infer_ct below.
 inferConstraints :: forall q. DsMonad q => DPred -> DType -> [DCon] -> q DCxt
-inferConstraints pr inst_ty = fmap (nubBy geq) . concatMapM infer_ct
+inferConstraints pr inst_ty = fmap nub . concatMapM infer_ct
   where
     -- A thorny situation arises when attempting to infer an instance context
     -- for a GADT. Consider the following example:
@@ -128,7 +126,7 @@ inferConstraints pr inst_ty = fmap (nubBy geq) . concatMapM infer_ct
                       Just subst -> traverse (substTy subst) field_tys
       if is_functor_like
          then mk_functor_like_constraints field_tys' res_ty'
-         else pure $ map (pr `DAppPr`) field_tys'
+         else pure $ map (pr `DAppT`) field_tys'
 
     -- If we derive a Functor-like class, e.g.,
     --
@@ -142,15 +140,15 @@ inferConstraints pr inst_ty = fmap (nubBy geq) . concatMapM infer_ct
       -- This function is partial. But that's OK, because
       -- functorLikeValidityChecks ensures that this is total by the time
       -- we invoke this.
-      let _ :| res_ty_args     = unfoldType res_ty
-          (_, last_res_ty_arg) = snocView res_ty_args
+      let (_, res_ty_args)     = unfoldDType res_ty
+          (_, last_res_ty_arg) = snocView $ filterDTANormals res_ty_args
           Just last_tv         = getDVarTName_maybe last_res_ty_arg
       deep_subtypes <- concatMapM (deepSubtypesContaining last_tv) fields
-      pure $ map (pr `DAppPr`) deep_subtypes
+      pure $ map (pr `DAppT`) deep_subtypes
 
     is_functor_like :: Bool
     is_functor_like
-      | DConT pr_class_name :| _ <- unfoldType (predToType pr)
+      | (DConT pr_class_name, _) <- unfoldDType pr
       = isFunctorLikeClassName pr_class_name
       | otherwise
       = False
