@@ -18,6 +18,8 @@ import Language.Haskell.TH.Desugar
 import GHC.TypeLits ( Nat, Symbol )
 import GHC.Exts ( Constraint )
 import GHC.Show ( showCommaSpace, showSpace )
+import Data.Type.Equality ( TestEquality(..) )
+import Data.Type.Coercion ( TestCoercion(..) )
 import Data.Typeable ( TypeRep )
 import Data.Singletons.Util
 import Control.Applicative
@@ -26,13 +28,16 @@ import Control.Monad
 boolName, andName, tyEqName, compareName, minBoundName,
   maxBoundName, repName,
   nilName, consName, listName, tyFunArrowName,
-  applyName, natName, symbolName, typeRepName, stringName,
+  applyName, applyTyConName, applyTyConAux1Name,
+  natName, symbolName, typeRepName, stringName,
   eqName, ordName, boundedName, orderingName,
-  singFamilyName, singIName, singMethName, demoteName,
+  singFamilyName, singIName, singMethName, demoteName, withSingIName,
   singKindClassName, sEqClassName, sEqMethName, sconsName, snilName, strueName,
   sIfName,
   someSingTypeName, someSingDataName,
   sListName, sDecideClassName, sDecideMethName,
+  testEqualityClassName, testEqualityMethName, decideEqualityName,
+  testCoercionClassName, testCoercionMethName, decideCoercionName,
   provedName, disprovedName, reflName, toSingName, fromSingName,
   equalityName, applySingName, suppressClassName, suppressMethodName,
   thenCmpName,
@@ -40,8 +45,8 @@ boolName, andName, tyEqName, compareName, minBoundName,
   sNegateName, errorName, foldlName, cmpEQName, cmpLTName, cmpGTName,
   singletonsToEnumName, singletonsFromEnumName, enumName, singletonsEnumName,
   equalsName, constraintName,
-  showName, showCharName, showCommaSpaceName, showParenName, showsPrecName,
-  showSpaceName, showStringName, showSingName,
+  showName, showSName, showCharName, showCommaSpaceName, showParenName, showsPrecName,
+  showSpaceName, showStringName, showSingName, showSing'Name,
   composeName, gtName, tyFromStringName, sFromStringName,
   foldableName, foldMapName, memptyName, mappendName, foldrName,
   functorName, fmapName, replaceName,
@@ -58,6 +63,8 @@ consName = '(:)
 listName = ''[]
 tyFunArrowName = ''(~>)
 applyName = ''Apply
+applyTyConName = ''ApplyTyCon
+applyTyConAux1Name = ''ApplyTyConAux1
 symbolName = ''Symbol
 natName = ''Nat
 typeRepName = ''TypeRep
@@ -72,6 +79,7 @@ singMethName = 'sing
 toSingName = 'toSing
 fromSingName = 'fromSing
 demoteName = ''Demote
+withSingIName = 'withSingI
 singKindClassName = ''SingKind
 sEqClassName = mk_name_tc "Data.Singletons.Prelude.Eq" "SEq"
 sEqMethName = mk_name_v "Data.Singletons.Prelude.Eq" "%=="
@@ -84,6 +92,12 @@ someSingDataName = 'SomeSing
 sListName = mk_name_tc "Data.Singletons.Prelude.Instances" "SList"
 sDecideClassName = ''SDecide
 sDecideMethName = '(%~)
+testEqualityClassName = ''TestEquality
+testEqualityMethName = 'testEquality
+decideEqualityName = 'decideEquality
+testCoercionClassName = ''TestCoercion
+testCoercionMethName = 'testCoercion
+decideCoercionName = 'decideCoercion
 provedName = 'Proved
 disprovedName = 'Disproved
 reflName = 'Refl
@@ -109,12 +123,14 @@ singletonsEnumName = mk_name_tc "Data.Singletons.Prelude.Enum" "Enum"
 equalsName = '(==)
 constraintName = ''Constraint
 showName = ''Show
+showSName = ''ShowS
 showCharName = 'showChar
 showParenName = 'showParen
 showSpaceName = 'showSpace
 showsPrecName = 'showsPrec
 showStringName = 'showString
 showSingName = mk_name_tc "Data.Singletons.ShowSing" "ShowSing"
+showSing'Name = mk_name_tc "Data.Singletons.ShowSing" "ShowSing'"
 composeName = '(.)
 gtName = '(>)
 showCommaSpaceName = 'showCommaSpace
@@ -161,13 +177,7 @@ promoteValNameLhs = promoteValNameLhsPrefix noPrefix
 
 -- like promoteValNameLhs, but adds a prefix to the promoted name
 promoteValNameLhsPrefix :: (String, String) -> Name -> Name
-promoteValNameLhsPrefix pres@(alpha, symb) n
-  | nameBase n == "."
-  = mkName $ symb ++ ":."
-  | nameBase n == "!"
-  = mkName $ symb ++ ":!"
-    -- See Note [Special cases for (.) and (!)]
-
+promoteValNameLhsPrefix pres@(alpha, _) n
     -- We can't promote promote idenitifers beginning with underscores to
     -- type names, so we work around the issue by prepending "US" at the
     -- front of the name (#229).
@@ -193,15 +203,6 @@ promoteValRhs name
 -- names.
 promoteTySym :: Name -> Int -> Name
 promoteTySym name sat
-    | nameBase name == ":."
-    = default_case (mkName ".")
-    | nameBase name == ":!"
-    = default_case (mkName "!")
-      -- Although (:.) and (:!) are special cases, we need not have a colon in
-      -- front of their defunctionalization symbols, since only the names
-      -- (.) and (!) are problematic for the parser.
-      -- See Note [Special cases for (.) and (!)]
-
       -- We can't promote promote idenitifers beginning with underscores to
       -- type names, so we work around the issue by prepending "US" at the
       -- front of the name (#229).
@@ -285,7 +286,7 @@ singFamily :: DType
 singFamily = DConT singFamilyName
 
 singKindConstraint :: DKind -> DPred
-singKindConstraint = DAppPr (DConPr singKindClassName)
+singKindConstraint = DAppT (DConT singKindClassName)
 
 demote :: DType
 demote = DConT demoteName
@@ -302,9 +303,9 @@ mkListE =
 foldApply :: DType -> [DType] -> DType
 foldApply = foldl apply
 
--- make and equality predicate
+-- make an equality predicate
 mkEqPred :: DType -> DType -> DPred
-mkEqPred ty1 ty2 = foldPred (DConPr equalityName) [ty1, ty2]
+mkEqPred ty1 ty2 = foldType (DConT equalityName) [ty1, ty2]
 
 -- | If a 'String' begins with one or more underscores, return
 -- @'Just' (us, rest)@, where @us@ contain all of the underscores at the
@@ -315,16 +316,19 @@ splitUnderscores s = case span (== '_') s of
                        ([], _) -> Nothing
                        res     -> Just res
 
--- Walk a DPred, applying a function to all occurrences of constructor names.
-modifyConNameDPred :: (Name -> Name) -> DPred -> DPred
-modifyConNameDPred mod_con_name = go
+-- Walk a DType, applying a function to all occurrences of constructor names.
+modifyConNameDType :: (Name -> Name) -> DType -> DType
+modifyConNameDType mod_con_name = go
   where
-    go (DForallPr tvbs cxt p) = DForallPr tvbs (map go cxt) (go p)
-    go (DAppPr p t)           = DAppPr (go p) t
-    go (DSigPr p k)           = DSigPr (go p) k
-    go p@(DVarPr _)           = p
-    go (DConPr n)             = DConPr (mod_con_name n)
-    go p@DWildCardPr          = p
+    go (DForallT tvbs cxt p) = DForallT tvbs (map go cxt) (go p)
+    go (DAppT     p t)       = DAppT     (go p) t
+    go (DAppKindT p k)       = DAppKindT (go p) k
+    go (DSigT     p k)       = DSigT     (go p) k
+    go p@(DVarT _)           = p
+    go (DConT n)             = DConT (mod_con_name n)
+    go p@DWildCardT          = p
+    go p@(DLitT {})          = p
+    go p@DArrowT             = p
 
 {-
 Note [Defunctionalization symbol suffixes]
@@ -351,12 +355,4 @@ $@#@$
 $@#@$$
 
 And there is no conflict.
-
-Note [Special cases for (.) and (!)]
-~~~~~~~~~~~~~~~~~~~~~~~~~~~
-Almost every infix value name can be promoted trivially. For example, (+) works
-both at the value- and type-level. The two exceptions to this rule are (.) and (!),
-which we promote to the special type names (:.) and (:!), respectively.
-This is necessary since one cannot define or apply (.) or (!) at the type level --
-they simply won't parse. Bummer.
 -}
