@@ -30,15 +30,15 @@ import Data.Singletons.Names
 import Language.Haskell.TH.Syntax hiding (showName)
 import Language.Haskell.TH.Ppr
 import Language.Haskell.TH.Desugar
+import qualified Language.Haskell.TH.Desugar.OMap.Strict as OMap
+import Language.Haskell.TH.Desugar.OMap.Strict (OMap)
 import Data.Singletons.Util
 
 import Control.Monad
 import Data.Bifunctor (bimap)
-import Data.List.NonEmpty (NonEmpty(..))
 import qualified Data.Map as Map
 import Data.Map (Map)
 import Data.Maybe
-import Data.Semigroup (Semigroup(..))
 
 data PartitionedDecs =
   PDecs { pd_let_decs :: [DLetDec]
@@ -77,7 +77,7 @@ partitionDec (DDataD _nd _cxt name tvbs mk cons derivings) = do
   derived_decs
     <- mapM (\(strat, deriv_pred) ->
               let etad_tvbs
-                    | DConT pred_name :| _ <- unfoldType deriv_pred
+                    | (DConT pred_name, _) <- unfoldDType deriv_pred
                     , isFunctorLikeClassName pred_name
                       -- If deriving Functor, Foldable, or Traversable,
                       -- we need to use one less type variable than we normally do.
@@ -89,9 +89,9 @@ partitionDec (DDataD _nd _cxt name tvbs mk cons derivings) = do
       $ concatMap flatten_clause derivings
   return $ mconcat $ derived_dec : derived_decs
   where
-    flatten_clause :: DDerivClause -> [(Maybe DDerivStrategy, DType)]
+    flatten_clause :: DDerivClause -> [(Maybe DDerivStrategy, DPred)]
     flatten_clause (DDerivClause strat preds) =
-      map (\p -> (strat, predToType p)) preds
+      map (\p -> (strat, p)) preds
 
 partitionDec (DClassD cxt name tvbs fds decs) = do
   (lde, otfs) <- concatMapM partitionClassDec decs
@@ -101,7 +101,7 @@ partitionDec (DClassD cxt name tvbs fds decs) = do
                                                , cd_fds       = fds
                                                , cd_lde       = lde }]
                   , pd_open_type_family_decs = otfs }
-partitionDec (DInstanceD _ cxt ty decs) = do
+partitionDec (DInstanceD _ _ cxt ty decs) = do
   (defns, sigs) <- liftM (bimap catMaybes mconcat) $
                    mapAndUnzipM partitionInstanceDec decs
   (name, tys) <- split_app_tys [] ty
@@ -116,9 +116,9 @@ partitionDec (DInstanceD _ cxt ty decs) = do
     split_app_tys acc (DSigT t _)   = split_app_tys acc t
     split_app_tys _ _ = fail $ "Illegal instance head: " ++ show ty
 partitionDec (DRoleAnnotD {}) = return mempty  -- ignore these
-partitionDec (DTySynD name tvbs _type) =
+partitionDec (DTySynD name tvbs rhs) =
   -- See Note [Partitioning, type synonyms, and type families]
-  pure $ mempty { pd_ty_syn_decs = [TySynDecl name tvbs] }
+  pure $ mempty { pd_ty_syn_decs = [TySynDecl name tvbs rhs] }
 partitionDec (DClosedTypeFamilyD tf_head _) =
   -- See Note [Partitioning, type synonyms, and type families]
   pure $ mempty { pd_closed_type_family_decs = [TypeFamilyDecl tf_head] }
@@ -128,13 +128,14 @@ partitionDec (DOpenTypeFamilyD tf_head) =
 partitionDec (DTySynInstD {}) = pure mempty
   -- There's no need to track type family instances, since
   -- we already record the type family itself separately.
-partitionDec (DStandaloneDerivD mb_strat ctxt ty) =
-  case unfoldType ty of
-    cls_pred_ty :| cls_tys
-      | not (null cls_tys) -- We can't handle zero-parameter type classes
-      , let cls_arg_tys  = init cls_tys
-            data_ty      = last cls_tys
-            data_ty_head = case unfoldType data_ty of ty_head :| _ -> ty_head
+partitionDec (DStandaloneDerivD mb_strat _ ctxt ty) =
+  case unfoldDType ty of
+    (cls_pred_ty, cls_tys)
+      | let cls_normal_tys = filterDTANormals cls_tys
+      , not (null cls_normal_tys) -- We can't handle zero-parameter type classes
+      , let cls_arg_tys  = init cls_normal_tys
+            data_ty      = last cls_normal_tys
+            data_ty_head = case unfoldDType data_ty of (ty_head, _) -> ty_head
       , DConT data_tycon <- data_ty_head -- We can't handle deriving an instance for something
                                          -- other than a type constructor application
       -> do let cls_pred = foldType cls_pred_ty cls_arg_tys
@@ -152,10 +153,10 @@ partitionDec (DStandaloneDerivD mb_strat ctxt ty) =
 partitionDec dec =
   fail $ "Declaration cannot be promoted: " ++ pprint (decToTH dec)
 
-partitionClassDec :: Monad m => DDec -> m (ULetDecEnv, [OpenTypeFamilyDecl])
+partitionClassDec :: MonadFail m => DDec -> m (ULetDecEnv, [OpenTypeFamilyDecl])
 partitionClassDec (DLetDec (DSigD name ty)) =
   pure (typeBinding name ty, mempty)
-partitionClassDec (DLetDec (DValD (DVarPa name) exp)) =
+partitionClassDec (DLetDec (DValD (DVarP name) exp)) =
   pure (valueBinding name (UValue exp), mempty)
 partitionClassDec (DLetDec (DFunD name clauses)) =
   pure (valueBinding name (UFunction clauses), mempty)
@@ -173,16 +174,16 @@ partitionClassDec (DTySynInstD {}) =
 partitionClassDec _ =
   fail "Only method declarations can be promoted within a class."
 
-partitionInstanceDec :: Monad m => DDec
+partitionInstanceDec :: MonadFail m => DDec
                      -> m ( Maybe (Name, ULetDecRHS) -- right-hand sides of methods
-                          , Map Name DType           -- method type signatures
+                          , OMap Name DType          -- method type signatures
                           )
-partitionInstanceDec (DLetDec (DValD (DVarPa name) exp)) =
+partitionInstanceDec (DLetDec (DValD (DVarP name) exp)) =
   pure (Just (name, UValue exp), mempty)
 partitionInstanceDec (DLetDec (DFunD name clauses)) =
   pure (Just (name, UFunction clauses), mempty)
 partitionInstanceDec (DLetDec (DSigD name ty)) =
-  pure (Nothing, Map.singleton name ty)
+  pure (Nothing, OMap.singleton name ty)
 partitionInstanceDec (DLetDec (DPragmaD {})) =
   pure (Nothing, mempty)
 partitionInstanceDec (DTySynInstD {}) =
@@ -196,7 +197,7 @@ partitionDeriving
   :: forall m. DsMonad m
   => Maybe DDerivStrategy
                 -- ^ The deriving strategy, if present.
-  -> DType      -- ^ The class being derived (e.g., 'Eq'), possibly applied to
+  -> DPred      -- ^ The class being derived (e.g., 'Eq'), possibly applied to
                 --   some number of arguments (e.g., @C Int Bool@).
   -> Maybe DCxt -- ^ @'Just' ctx@ if @ctx@ was provided via @StandaloneDeriving@.
                 --   'Nothing' if using a @deriving@ clause.
@@ -204,8 +205,8 @@ partitionDeriving
   -> DataDecl   -- ^ The original data type information (e.g., its constructors).
   -> m PartitionedDecs
 partitionDeriving mb_strat deriv_pred mb_ctxt ty data_decl =
-  case unfoldType deriv_pred of
-    DConT deriv_name :| arg_tys
+  case unfoldDType deriv_pred of
+    (DConT deriv_name, arg_tys)
          -- Here, we are more conservative than GHC: DeriveAnyClass only kicks
          -- in if the user explicitly chooses to do so with the anyclass
          -- deriving strategy
@@ -223,7 +224,7 @@ partitionDeriving mb_strat deriv_pred mb_ctxt ty data_decl =
                       -- StandaloneDeriving, use that.)
 
                     , id_name      = deriv_name
-                    , id_arg_tys   = arg_tys ++ [ty]
+                    , id_arg_tys   = filterDTANormals arg_tys ++ [ty]
                     , id_sigs      = mempty
                     , id_meths     = [] }
 
@@ -237,7 +238,7 @@ partitionDeriving mb_strat deriv_pred mb_ctxt ty data_decl =
 
     -- Stock classes. These are derived only if `singletons` supports them
     -- (and, optionally, if an explicit stock deriving strategy is used)
-    DConT deriv_name :| [] -- For now, all stock derivable class supported in
+    (DConT deriv_name, []) -- For now, all stock derivable class supported in
                            -- singletons take just one argument (the data
                            -- type itself)
        | stock_or_default
@@ -258,9 +259,17 @@ partitionDeriving mb_strat deriv_pred mb_ctxt ty data_decl =
 
       mk_derived_inst    dec = mempty { pd_instance_decs   = [dec] }
       mk_derived_eq_inst dec = mempty { pd_derived_eq_decs = [dec] }
-      derived_decl = DerivedDecl { ded_mb_cxt = mb_ctxt
-                                 , ded_type   = ty
-                                 , ded_decl   = data_decl }
+
+      derived_decl :: DerivedDecl cls
+      derived_decl = DerivedDecl { ded_mb_cxt     = mb_ctxt
+                                 , ded_type       = ty
+                                 , ded_type_tycon = ty_tycon
+                                 , ded_decl       = data_decl }
+        where
+          ty_tycon :: Name
+          ty_tycon = case unfoldDType ty of
+                       (DConT tc, _) -> tc
+                       (t,        _) -> error $ "Not a data type: " ++ show t
       stock_or_default = isStockOrDefault mb_strat
 
       -- A mapping from all stock derivable classes (that singletons supports)
@@ -277,7 +286,7 @@ partitionDeriving mb_strat deriv_pred mb_ctxt ty data_decl =
         , ( eqName, return $ mk_derived_eq_inst derived_decl )
           -- See Note [DerivedDecl] in Data.Singletons.Syntax
         , ( showName, do -- These will become PShow/SShow instances...
-                         inst_for_promotion <- mk_instance mkShowInstance
+                         inst_for_promotion <- mk_instance $ mkShowInstance ForPromotion
                          -- ...and this will become a Show instance.
                          let inst_for_show = derived_decl
                          pure $ mempty { pd_instance_decs     = [inst_for_promotion]
