@@ -22,15 +22,11 @@ module Data.Singletons.Prelude (
   -- * Basic singleton definitions
   module Data.Singletons,
 
-  Sing(SFalse, STrue, SNil, SCons, SJust, SNothing, SLeft, SRight, SLT, SEQ, SGT,
-       STuple0, STuple2, STuple3, STuple4, STuple5, STuple6, STuple7),
+  -- * Singleton types
 
-  -- * Singleton type synonyms
-
-  -- | These synonyms are all kind-restricted synonyms of 'Sing'.
-  -- For example 'SBool' requires an argument of kind 'Bool'.
-  SBool, SList, SMaybe, SEither, SOrdering,
-  STuple0, STuple2, STuple3, STuple4, STuple5, STuple6, STuple7,
+  SBool(..), SList(..), SMaybe(..), SEither(..), SOrdering(..),
+  STuple0(..), STuple2(..), STuple3(..), STuple4(..),
+  STuple5(..), STuple6(..), STuple7(..),
 
   -- * Functions working with 'Bool'
   If, sIf, Not, sNot, type (&&), type (||), (%&&), (%||), Otherwise, sOtherwise,
@@ -65,12 +61,14 @@ module Data.Singletons.Prelude (
   PSemigroup(type (<>)), SSemigroup((%<>)),
   PMonoid(..), SMonoid(..),
 
-  -- * Singleton 'Functor', 'Applicative', and 'Monad'
+  -- * Singleton 'Functor', 'Applicative', 'Monad', and 'MonadFail'
   PFunctor(Fmap, type (<$)), SFunctor(sFmap, (%<$)), type (<$>), (%<$>),
   PApplicative(Pure, type (<*>), type (*>), type (<*)),
   SApplicative(sPure, (%<*>), (%*>), (%<*)),
-  PMonad(type (>>=), type (>>), Return, Fail),
-  SMonad((%>>=), (%>>), sReturn, sFail),
+  PMonad(type (>>=), type (>>), Return),
+  SMonad((%>>=), (%>>), sReturn),
+  PMonadFail(Fail), SMonadFail(sFail),
+
   MapM_, sMapM_,
   Sequence_, sSequence_,
   type (=<<), (%=<<),
@@ -84,7 +82,7 @@ module Data.Singletons.Prelude (
   STraversable(sTraverse, sSequenceA, sMapM, sSequence),
 
   -- ** Miscellaneous functions
-  Id, sId, Const, sConst, (:.), (%.), type ($), (%$), type ($!), (%$!),
+  Id, sId, Const, sConst, type (.), (%.), type ($), (%$), type ($!), (%$!),
   Flip, sFlip, AsTypeOf, sAsTypeOf,
   Seq, sSeq,
 
