@@ -1,5 +1,6 @@
 {-# LANGUAGE TemplateHaskell, ScopedTypeVariables, TypeFamilies, GADTs,
-             RankNTypes, UndecidableInstances, DataKinds, PolyKinds #-}
+             RankNTypes, UndecidableInstances, DataKinds, PolyKinds,
+             TypeApplications #-}
 
 -----------------------------------------------------------------------------
 -- |
@@ -23,16 +24,7 @@
 
 module Data.Singletons.Prelude.Either (
   -- * The 'Either' singleton
-  Sing(SLeft, SRight),
-  -- | Though Haddock doesn't show it, the 'Sing' instance above declares
-  -- constructors
-  --
-  -- > SLeft  :: Sing a -> Sing (Left a)
-  -- > SRight :: Sing b -> Sing (Right b)
-
-  SEither,
-  -- | 'SEither' is a kind-restricted synonym for 'Sing':
-  -- @type SEither (a :: Either x y) = Sing a@
+  Sing, SEither(..),
 
   -- * Singletons from @Data.Either@
   either_, Either_, sEither_,
