@@ -19,11 +19,11 @@
 
 {-# LANGUAGE TemplateHaskell, ScopedTypeVariables, TypeFamilies,
              TypeOperators, UndecidableInstances, GADTs,
-             DataKinds, PolyKinds #-}
+             DataKinds, PolyKinds, TypeApplications #-}
 
 module Data.Singletons.Prelude.Function (
     -- * "Prelude" re-exports
-    Id, sId, Const, sConst, (:.), (%.), Flip, sFlip, type ($), (%$)
+    Id, sId, Const, sConst, type (.), (%.), Flip, sFlip, type ($), (%$)
     -- * Other combinators
   , type (&), (%&), On, sOn
 
