@@ -14,6 +14,10 @@ module Data.Singletons.Promote where
 import Language.Haskell.TH hiding ( Q, cxt )
 import Language.Haskell.TH.Syntax ( Quasi(..) )
 import Language.Haskell.TH.Desugar
+import qualified Language.Haskell.TH.Desugar.OMap.Strict as OMap
+import Language.Haskell.TH.Desugar.OMap.Strict (OMap)
+import qualified Language.Haskell.TH.Desugar.OSet as OSet
+import Language.Haskell.TH.Desugar.OSet (OSet)
 import Data.Singletons.Names
 import Data.Singletons.Promote.Monad
 import Data.Singletons.Promote.Eq
@@ -31,13 +35,10 @@ import Prelude hiding (exp)
 import Control.Applicative (Alternative(..))
 import Control.Arrow (second)
 import Control.Monad
-import Control.Monad.Trans.Class (MonadTrans(..))
 import Control.Monad.Trans.Maybe
 import Control.Monad.Writer
 import qualified Data.Map.Strict as Map
 import Data.Map.Strict ( Map )
-import qualified Data.Set as Set
-import Data.Set ( Set )
 import Data.Maybe
 import qualified GHC.LanguageExtensions.Type as LangExt
 
@@ -141,7 +142,7 @@ promoteShowInstances = concatMapM promoteShowInstance
 
 -- | Produce an instance for 'PShow' from the given type
 promoteShowInstance :: DsMonad q => Name -> q [Dec]
-promoteShowInstance = promoteInstance mkShowInstance "Show"
+promoteShowInstance = promoteInstance (mkShowInstance ForPromotion) "Show"
 
 -- | Produce an instance for @(==)@ (type-level equality) from the given type
 promoteEqInstance :: DsMonad q => Name -> q [Dec]
@@ -163,7 +164,7 @@ promoteInstance mk_inst class_name name = do
   cons' <- concatMapM (dsCon tvbs' data_ty) cons
   let data_decl = DataDecl name tvbs' cons'
   raw_inst <- mk_inst Nothing data_ty data_decl
-  decs <- promoteM_ [] $ void $ promoteInstanceDec Map.empty raw_inst
+  decs <- promoteM_ [] $ void $ promoteInstanceDec OMap.empty raw_inst
   return $ decsToTH decs
 
 promoteInfo :: DInfo -> PrM ()
@@ -256,12 +257,12 @@ promoteLetDecs prefixes decls = do
   let_dec_env <- buildLetDecEnv decls
   all_locals <- allLocals
   let binds = [ (name, foldType (DConT sym) (map DVarT all_locals))
-              | name <- Map.keys $ lde_defns let_dec_env
+              | (name, _) <- OMap.assocs $ lde_defns let_dec_env
               , let proName = promoteValNameLhsPrefix prefixes name
                     sym = promoteTySym proName (length all_locals) ]
   (decs, let_dec_env') <- letBind binds $ promoteLetDecEnv prefixes let_dec_env
   emitDecs decs
-  return (binds, let_dec_env' { lde_proms = Map.fromList binds })
+  return (binds, let_dec_env' { lde_proms = OMap.fromList binds })
 
 -- Promotion of data types to kinds is automatic (see "Giving Haskell a
 -- Promotion" paper for more details). Here we "plug into" the promotion
@@ -297,8 +298,7 @@ promoteDataDec (DataDecl _name _tvbs ctors) = do
 
 promoteClassDec :: UClassDecl
                 -> PrM AClassDecl
-promoteClassDec decl@(ClassDecl { cd_cxt  = cxt
-                                , cd_name = cls_name
+promoteClassDec decl@(ClassDecl { cd_name = cls_name
                                 , cd_tvbs = tvbs'
                                 , cd_fds  = fundeps
                                 , cd_lde  = lde@LetDecEnv
@@ -309,31 +309,30 @@ promoteClassDec decl@(ClassDecl { cd_cxt  = cxt
     -- a workaround for GHC Trac #12928; see Note [CUSKification]
     tvbs = map cuskify tvbs'
   let pClsName = promoteClassName cls_name
-  pCxt <- mapM promote_superclass_pred cxt
   forallBind cls_kvs_to_bind $ do
-    sig_decs <- mapM (uncurry promote_sig) (Map.toList meth_sigs)
-    let defaults_list  = Map.toList defaults
+    sig_decs <- mapM (uncurry promote_sig) (OMap.assocs meth_sigs)
+    let defaults_list  = OMap.assocs defaults
         defaults_names = map fst defaults_list
     (default_decs, ann_rhss, prom_rhss)
-      <- mapAndUnzip3M (promoteMethod Map.empty Nothing meth_sigs) defaults_list
+      <- mapAndUnzip3M (promoteMethod OMap.empty Nothing meth_sigs) defaults_list
 
     let infix_decls' = catMaybes $ map (uncurry promoteInfixDecl)
-                                 $ Map.toList infix_decls
+                                 $ OMap.assocs infix_decls
 
     -- no need to do anything to the fundeps. They work as is!
-    emitDecs [DClassD pCxt pClsName tvbs fundeps
+    emitDecs [DClassD [] pClsName tvbs fundeps
                       (sig_decs ++ default_decs ++ infix_decls')]
     let defaults_list'   = zip defaults_names ann_rhss
         proms            = zip defaults_names prom_rhss
         cls_kvs_to_bind' = cls_kvs_to_bind <$ meth_sigs
-    return (decl { cd_lde = lde { lde_defns     = Map.fromList defaults_list'
-                                , lde_proms     = Map.fromList proms
+    return (decl { cd_lde = lde { lde_defns     = OMap.fromList defaults_list'
+                                , lde_proms     = OMap.fromList proms
                                 , lde_bound_kvs = cls_kvs_to_bind' } })
   where
-    cls_kvb_names, cls_tvb_names, cls_kvs_to_bind :: Set Name
+    cls_kvb_names, cls_tvb_names, cls_kvs_to_bind :: OSet Name
     cls_kvb_names   = foldMap (foldMap fvDType . extractTvbKind) tvbs'
-    cls_tvb_names   = Set.fromList $ map extractTvbName tvbs'
-    cls_kvs_to_bind = cls_kvb_names `Set.union` cls_tvb_names
+    cls_tvb_names   = OSet.fromList $ map extractTvbName tvbs'
+    cls_kvs_to_bind = cls_kvb_names `OSet.union` cls_tvb_names
 
     promote_sig :: Name -> DType -> PrM DDec
     promote_sig name ty = do
@@ -348,19 +347,8 @@ promoteClassDec decl@(ClassDecl { cd_cxt  = cxt
                                                  (DKindSig resK)
                                                  Nothing)
 
-    promote_superclass_pred :: DPred -> PrM DPred
-    promote_superclass_pred = go
-      where
-      go (DForallPr {}) = fail "Cannot promote quantified constraints"
-      go (DAppPr pr ty) = DAppPr <$> go pr <*> promoteType ty
-      go (DSigPr pr _k) = go pr    -- just ignore the kind; it can't matter
-      go (DVarPr name)  = fail $ "Cannot promote ConstraintKinds variables like "
-                              ++ show name
-      go (DConPr name)  = return $ DConPr (promoteClassName name)
-      go DWildCardPr    = return DWildCardPr
-
 -- returns (unpromoted method name, ALetDecRHS) pairs
-promoteInstanceDec :: Map Name DType -> UInstDecl -> PrM AInstDecl
+promoteInstanceDec :: OMap Name DType -> UInstDecl -> PrM AInstDecl
 promoteInstanceDec orig_meth_sigs
                    decl@(InstDecl { id_name     = cls_name
                                   , id_arg_tys  = inst_tys
@@ -372,8 +360,8 @@ promoteInstanceDec orig_meth_sigs
   forallBind kvs_to_bind $ do
     let subst = Map.fromList $ zip cls_tvb_names inst_kis
     (meths', ann_rhss, _) <- mapAndUnzip3M (promoteMethod inst_sigs (Just subst) orig_meth_sigs) meths
-    emitDecs [DInstanceD Nothing [] (foldType (DConT pClsName)
-                                      inst_kis) meths']
+    emitDecs [DInstanceD Nothing Nothing [] (foldType (DConT pClsName)
+                                              inst_kis) meths']
     return (decl { id_meths = zip (map fst meths) ann_rhss })
   where
     pClsName = promoteClassName cls_name
@@ -430,19 +418,16 @@ dsReifyTypeNameInfo, which first calls lookupTypeName (to ensure we can find a N
 that's in the type namespace) and _then_ reifies it.
 -}
 
-promoteMethod :: Map Name DType -- InstanceSigs for methods
+promoteMethod :: OMap Name DType -- InstanceSigs for methods
               -> Maybe (Map Name DKind)
                     -- ^ instantiations for class tyvars (Nothing for default decls)
                     --   See Note [Promoted class method kinds]
-              -> Map Name DType     -- method types
+              -> OMap Name DType    -- method types
               -> (Name, ULetDecRHS)
               -> PrM (DDec, ALetDecRHS, DType)
                  -- returns (type instance, ALetDecRHS, promoted RHS)
 promoteMethod inst_sigs_map m_subst orig_sigs_map (meth_name, meth_rhs) = do
   (meth_arg_kis, meth_res_ki) <- lookup_meth_ty
-  ((_, _, _, eqns), _defuns, ann_rhs)
-    <- promoteLetDecRHS (Just (meth_arg_kis, meth_res_ki)) Map.empty Map.empty
-                        noPrefix meth_name meth_rhs
   meth_arg_tvs <- mapM (const $ qNewName "a") meth_arg_kis
   let helperNameBase = case nameBase proName of
                          first:_ | not (isHsLetter first) -> "TFHelper"
@@ -462,17 +447,21 @@ promoteMethod inst_sigs_map m_subst orig_sigs_map (meth_name, meth_rhs) = do
       -- strictly necessary, as kind inference can figure them out just as well.
       family_args = map DVarT meth_arg_tvs
   helperName <- newUniqueName helperNameBase
+  let proHelperName = promoteValNameLhs helperName
+  ((_, _, _, eqns), _defuns, ann_rhs)
+    <- promoteLetDecRHS (Just (meth_arg_kis, meth_res_ki)) OMap.empty OMap.empty
+                        noPrefix helperName meth_rhs
   let tvbs = zipWith DKindedTV meth_arg_tvs meth_arg_kis
   emitDecs [DClosedTypeFamilyD (DTypeFamilyHead
-                                  helperName
+                                  proHelperName
                                   tvbs
                                   (DKindSig meth_res_ki)
                                   Nothing)
                                eqns]
-  emitDecsM (defunctionalize helperName Nothing tvbs (Just meth_res_ki))
+  emitDecsM (defunctionalize proHelperName Nothing tvbs (Just meth_res_ki))
   return ( DTySynInstD
-             proName
-             (DTySynEqn family_args
+             (DTySynEqn Nothing
+                        (foldType (DConT proName) family_args)
                         (foldApply (promoteValRhs helperName) (map DVarT meth_arg_tvs)))
          , ann_rhs
          , DConT (promoteTySym helperName 0) )
@@ -481,7 +470,7 @@ promoteMethod inst_sigs_map m_subst orig_sigs_map (meth_name, meth_rhs) = do
 
     lookup_meth_ty :: PrM ([DKind], DKind)
     lookup_meth_ty =
-      case Map.lookup meth_name inst_sigs_map of
+      case OMap.lookup meth_name inst_sigs_map of
         Just ty ->
           -- We have an InstanceSig. These are easy: no substitution for clas
           -- variables is required at all!
@@ -490,7 +479,7 @@ promoteMethod inst_sigs_map m_subst orig_sigs_map (meth_name, meth_rhs) = do
           -- We don't have an InstanceSig, so we must compute the kind to use
           -- ourselves (possibly substituting for class variables below).
           (arg_kis, res_ki) <-
-            case Map.lookup meth_name orig_sigs_map of
+            case OMap.lookup meth_name orig_sigs_map of
               Nothing -> do
                 mb_info <- dsReifyTypeNameInfo proName
                            -- See Note [Using dsReifyTypeNameInfo when promoting instances]
@@ -551,10 +540,10 @@ promoteLetDecEnv prefixes (LetDecEnv { lde_defns = value_env
                                      , lde_types = type_env
                                      , lde_infix = fix_env }) = do
   let infix_decls = catMaybes $ map (uncurry promoteInfixDecl)
-                              $ Map.toList fix_env
+                              $ OMap.assocs fix_env
 
     -- promote all the declarations, producing annotated declarations
-  let (names, rhss) = unzip $ Map.toList value_env
+  let (names, rhss) = unzip $ OMap.assocs value_env
   (payloads, defun_decss, ann_rhss)
     <- fmap unzip3 $ zipWithM (promoteLetDecRHS Nothing type_env fix_env prefixes) names rhss
 
@@ -563,11 +552,11 @@ promoteLetDecEnv prefixes (LetDecEnv { lde_defns = value_env
   let decs = map payload_to_dec payloads ++ infix_decls
 
     -- build the ALetDecEnv
-  let let_dec_env' = LetDecEnv { lde_defns     = Map.fromList $ zip names ann_rhss
+  let let_dec_env' = LetDecEnv { lde_defns     = OMap.fromList $ zip names ann_rhss
                                , lde_types     = type_env
                                , lde_infix     = fix_env
-                               , lde_proms     = Map.empty   -- filled in promoteLetDecs
-                               , lde_bound_kvs = Map.fromList $ map (, bound_kvs) names }
+                               , lde_proms     = OMap.empty  -- filled in promoteLetDecs
+                               , lde_bound_kvs = OMap.fromList $ map (, bound_kvs) names }
 
   return (decs, let_dec_env')
   where
@@ -583,11 +572,8 @@ promoteInfixDecl name fixity
    -- If a name and its promoted counterpart are the same (modulo module
    -- prefixes), then there's no need to promote a fixity declaration for
    -- that name, since the existing fixity declaration will cover both
-   -- the term- and type-level versions of that name,
-   --
-   -- Names that fall into this category include data constructor names
-   -- and infix names, with the exceptions of (.) and (!).
-   -- See Note [Special cases for (.) and (!)] in Data.Singletons.Names.
+   -- the term- and type-level versions of that name. Names that fall into this
+   -- category include data constructor names and infix names.
  = Nothing
  | otherwise
  = Just $ DLetDec $ DInfixD fixity promoted_name
@@ -599,8 +585,8 @@ promoteInfixDecl name fixity
 -- an intermediate structure. Perhaps a better design is available.
 promoteLetDecRHS :: Maybe ([DKind], DKind)  -- the promoted type of the RHS (if known)
                                             -- needed to fix #136
-                 -> Map Name DType       -- local type env't
-                 -> Map Name Fixity      -- local fixity env't
+                 -> OMap Name DType      -- local type env't
+                 -> OMap Name Fixity     -- local fixity env't
                  -> (String, String)     -- let-binding prefixes
                  -> Name                 -- name of the thing being promoted
                  -> ULetDecRHS           -- body of the thing
@@ -612,7 +598,7 @@ promoteLetDecRHS m_rhs_ki type_env fix_env prefixes name (UValue exp) = do
     <- case m_rhs_ki of
          Just (arg_kis, res_ki) -> return ( Just (ravelTyFun (arg_kis ++ [res_ki]))
                                           , length arg_kis )
-         _ |  Just ty <- Map.lookup name type_env
+         _ |  Just ty <- OMap.lookup name type_env
            -> do ki <- promoteType ty
                  return (Just ki, countArgs ty)
            |  otherwise
@@ -623,18 +609,18 @@ promoteLetDecRHS m_rhs_ki type_env fix_env prefixes name (UValue exp) = do
       let lde_kvs_to_bind = foldMap fvDType res_kind
       (exp', ann_exp) <- forallBind lde_kvs_to_bind $ promoteExp exp
       let proName = promoteValNameLhsPrefix prefixes name
-          m_fixity = Map.lookup name fix_env
+          m_fixity = OMap.lookup name fix_env
           tvbs = map DPlainTV all_locals
       defuns <- defunctionalize proName m_fixity tvbs res_kind
       return ( ( proName, tvbs, res_kind
-               , [DTySynEqn (map DVarT all_locals) exp'] )
+               , [DTySynEqn Nothing (foldType (DConT proName) $ map DVarT all_locals) exp'] )
              , defuns
              , AValue (foldType (DConT proName) (map DVarT all_locals))
                       num_arrows ann_exp )
     _ -> do
       names <- replicateM num_arrows (newUniqueName "a")
-      let pats    = map DVarPa names
-          newArgs = map DVarE  names
+      let pats    = map DVarP names
+          newArgs = map DVarE names
       promoteLetDecRHS m_rhs_ki type_env fix_env prefixes name
                        (UFunction [DClause pats (foldExp exp newArgs)])
 
@@ -642,7 +628,7 @@ promoteLetDecRHS m_rhs_ki type_env fix_env prefixes name (UFunction clauses) = d
   numArgs <- count_args clauses
   (m_argKs, m_resK, ty_num_args) <- case m_rhs_ki of
     Just (arg_kis, res_ki) -> return (map Just arg_kis, Just res_ki, length arg_kis)
-    _ |  Just ty <- Map.lookup name type_env
+    _ |  Just ty <- OMap.lookup name type_env
       -> do
       -- promoteType turns arrows into TyFun. So, we unravel first to
       -- avoid this behavior. Note the use of ravelTyFun in resultK
@@ -654,7 +640,7 @@ promoteLetDecRHS m_rhs_ki type_env fix_env prefixes name (UFunction clauses) = d
       |  otherwise
       -> return (replicate numArgs Nothing, Nothing, numArgs)
   let proName  = promoteValNameLhsPrefix prefixes name
-      m_fixity = Map.lookup name fix_env
+      m_fixity = OMap.lookup name fix_env
   all_locals <- allLocals
   let local_tvbs = map DPlainTV all_locals
   tyvarNames <- mapM (const $ qNewName "a") m_argKs
@@ -664,7 +650,7 @@ promoteLetDecRHS m_rhs_ki type_env fix_env prefixes name (UFunction clauses) = d
   expClauses <- mapM (etaContractOrExpand ty_num_args numArgs) clauses
   let lde_kvs_to_bind = foldMap (foldMap fvDType) m_argKs <> foldMap fvDType m_resK
   (eqns, ann_clauses) <- forallBind lde_kvs_to_bind $
-                         mapAndUnzipM promoteClause expClauses
+                         mapAndUnzipM (promoteClause proName) expClauses
   prom_fun <- lookupVarE name
   return ( (proName, all_args, m_resK, eqns)
          , defun_decs
@@ -675,8 +661,8 @@ promoteLetDecRHS m_rhs_ki type_env fix_env prefixes name (UFunction clauses) = d
     etaContractOrExpand ty_num_args clause_num_args (DClause pats exp)
       | n >= 0 = do -- Eta-expand
           names <- replicateM n (newUniqueName "a")
-          let newPats = map DVarPa names
-              newArgs = map DVarE  names
+          let newPats = map DVarP names
+              newArgs = map DVarE names
           return $ DClause (pats ++ newPats) (foldExp exp newArgs)
       | otherwise = do -- Eta-contract
           let (clausePats, lamPats) = splitAt ty_num_args pats
@@ -688,8 +674,8 @@ promoteLetDecRHS m_rhs_ki type_env fix_env prefixes name (UFunction clauses) = d
     count_args (DClause pats _ : _) = return $ length pats
     count_args _ = fail $ "Impossible! A function without clauses."
 
-promoteClause :: DClause -> PrM (DTySynEqn, ADClause)
-promoteClause (DClause pats exp) = do
+promoteClause :: Name -> DClause -> PrM (DTySynEqn, ADClause)
+promoteClause proName (DClause pats exp) = do
   -- promoting the patterns creates variable bindings. These are passed
   -- to the function promoted the RHS
   ((types, pats'), prom_pat_infos) <- evalForPair $ mapAndUnzipM promotePat pats
@@ -699,11 +685,11 @@ promoteClause (DClause pats exp) = do
                    lambdaBind new_vars $
                    promoteExp exp
   all_locals <- allLocals   -- these are bound *outside* of this clause
-  return ( DTySynEqn (map DVarT all_locals ++ types) ty
+  return ( DTySynEqn Nothing (foldType (DConT proName) $ map DVarT all_locals ++ types) ty
          , ADClause new_vars pats' ann_exp )
 
-promoteMatch :: DMatch -> PrM (DTySynEqn, ADMatch)
-promoteMatch (DMatch pat exp) = do
+promoteMatch :: Name -> DMatch -> PrM (DTySynEqn, ADMatch)
+promoteMatch caseTFName (DMatch pat exp) = do
   -- promoting the patterns creates variable bindings. These are passed
   -- to the function promoted the RHS
   ((ty, pat'), prom_pat_infos) <- evalForPair $ promotePat pat
@@ -713,32 +699,34 @@ promoteMatch (DMatch pat exp) = do
                     lambdaBind new_vars $
                     promoteExp exp
   all_locals <- allLocals
-  return $ ( DTySynEqn (map DVarT all_locals ++ [ty]) rhs
+  return $ ( DTySynEqn Nothing
+                       (foldType (DConT caseTFName) $ map DVarT all_locals ++ [ty])
+                       rhs
            , ADMatch new_vars pat' ann_exp)
 
 -- promotes a term pattern into a type pattern, accumulating bound variable names
 promotePat :: DPat -> QWithAux PromDPatInfos PrM (DType, ADPat)
-promotePat (DLitPa lit) = (, ADLitPa lit) <$> promoteLitPat lit
-promotePat (DVarPa name) = do
+promotePat (DLitP lit) = (, ADLitP lit) <$> promoteLitPat lit
+promotePat (DVarP name) = do
       -- term vars can be symbols... type vars can't!
   tyName <- mkTyName name
-  tell $ PromDPatInfos [(name, tyName)] Set.empty
-  return (DVarT tyName, ADVarPa name)
-promotePat (DConPa name pats) = do
+  tell $ PromDPatInfos [(name, tyName)] OSet.empty
+  return (DVarT tyName, ADVarP name)
+promotePat (DConP name pats) = do
   (types, pats') <- mapAndUnzipM promotePat pats
   let name' = unboxed_tuple_to_tuple name
-  return (foldType (DConT name') types, ADConPa name pats')
+  return (foldType (DConT name') types, ADConP name pats')
   where
     unboxed_tuple_to_tuple n
       | Just deg <- unboxedTupleNameDegree_maybe n = tupleDataName deg
       | otherwise                                  = n
-promotePat (DTildePa pat) = do
+promotePat (DTildeP pat) = do
   qReportWarning "Lazy pattern converted into regular pattern in promotion"
-  second ADTildePa <$> promotePat pat
-promotePat (DBangPa pat) = do
+  second ADTildeP <$> promotePat pat
+promotePat (DBangP pat) = do
   qReportWarning "Strict pattern converted into regular pattern in promotion"
-  second ADBangPa <$> promotePat pat
-promotePat (DSigPa pat ty) = do
+  second ADBangP <$> promotePat pat
+promotePat (DSigP pat ty) = do
   -- We must maintain the invariant that any promoted pattern signature must
   -- not have any wildcards in the underlying pattern.
   -- See Note [Singling pattern signatures].
@@ -746,8 +734,8 @@ promotePat (DSigPa pat ty) = do
   (promoted, pat') <- promotePat wildless_pat
   ki <- promoteType ty
   tell $ PromDPatInfos [] (fvDType ki)
-  return (DSigT promoted ki, ADSigPa promoted pat' ki)
-promotePat DWildPa = return (DWildCardT, ADWildPa)
+  return (DSigT promoted ki, ADSigP promoted pat' ki)
+promotePat DWildP = return (DWildCardT, ADWildP)
 
 promoteExp :: DExp -> PrM (DType, ADExp)
 promoteExp (DVarE name) = fmap (, ADVarE name) $ lookupVarE name
@@ -775,7 +763,9 @@ promoteExp (DLamE names exp) = do
                                  tvbs
                                  DNoSig
                                  Nothing)
-                               [DTySynEqn (map DVarT (all_locals ++ tyNames))
+                               [DTySynEqn Nothing
+                                          (foldType (DConT lambdaName) $
+                                           map DVarT (all_locals ++ tyNames))
                                           rhs]]
   emitDecsM $ defunctionalize lambdaName Nothing tvbs Nothing
   let promLambda = foldl apply (DConT (promoteTySym lambdaName 0))
@@ -786,7 +776,7 @@ promoteExp (DCaseE exp matches) = do
   all_locals <- allLocals
   let prom_case = foldType (DConT caseTFName) (map DVarT all_locals)
   (exp', ann_exp)     <- promoteExp exp
-  (eqns, ann_matches) <- mapAndUnzipM promoteMatch matches
+  (eqns, ann_matches) <- mapAndUnzipM (promoteMatch caseTFName) matches
   tyvarName  <- qNewName "t"
   let all_args = all_locals ++ [tyvarName]
       tvbs     = map DPlainTV all_args
@@ -804,7 +794,7 @@ promoteExp (DLetE decs exp) = do
 promoteExp (DSigE exp ty) = do
   (exp', ann_exp) <- promoteExp exp
   ty' <- promoteType ty
-  return (DSigT exp' ty', ADSigE exp' ann_exp ty)
+  return (DSigT exp' ty', ADSigE exp' ann_exp ty')
 promoteExp e@(DStaticE _) = fail ("Static expressions cannot be promoted: " ++ show e)
 
 promoteLitExp :: Quasi q => Lit -> q DType
@@ -821,7 +811,7 @@ promoteLitExp (StringL str) = do
 promoteLitExp lit =
   fail ("Only string and natural number literals can be promoted: " ++ show lit)
 
-promoteLitPat :: Monad m => Lit -> m DType
+promoteLitPat :: MonadFail m => Lit -> m DType
 promoteLitPat (IntegerL n)
   | n >= 0    = return $ (DLitT (NumTyLit n))
   | otherwise =
