@@ -11,6 +11,7 @@ This file creates defunctionalization symbols for types during promotion.
 module Data.Singletons.Promote.Defun where
 
 import Language.Haskell.TH.Desugar
+import qualified Language.Haskell.TH.Desugar.OSet as OSet
 import Data.Singletons.Promote.Monad
 import Data.Singletons.Promote.Type
 import Data.Singletons.Names
@@ -18,10 +19,10 @@ import Language.Haskell.TH.Syntax
 import Data.Singletons.Syntax
 import Data.Singletons.Util
 import Control.Monad
+import Data.Foldable
 import qualified Data.Map.Strict as Map
 import Data.Map.Strict (Map)
 import Data.Maybe
-import qualified Data.Set as Set
 
 defunInfo :: DInfo -> PrM [DDec]
 defunInfo (DTyConI dec _instances) = buildDefunSyms dec
@@ -41,7 +42,7 @@ defunTypeDecls :: [TySynDecl]
                -> PrM ()
 defunTypeDecls ty_syns c_tyfams o_tyfams = do
   defun_ty_syns <-
-    concatMapM (\(TySynDecl name tvbs) -> buildDefunSymsTySynD name tvbs) ty_syns
+    concatMapM (\(TySynDecl name tvbs rhs) -> buildDefunSymsTySynD name tvbs rhs) ty_syns
   defun_c_tyfams <-
     concatMapM (buildDefunSymsClosedTypeFamilyD . getTypeFamilyDecl) c_tyfams
   defun_o_tyfams <-
@@ -55,8 +56,8 @@ buildDefunSyms (DClosedTypeFamilyD tf_head _) =
   buildDefunSymsClosedTypeFamilyD tf_head
 buildDefunSyms (DOpenTypeFamilyD tf_head) =
   buildDefunSymsOpenTypeFamilyD tf_head
-buildDefunSyms (DTySynD name tvbs _type) =
-  buildDefunSymsTySynD name tvbs
+buildDefunSyms (DTySynD name tvbs rhs) =
+  buildDefunSymsTySynD name tvbs rhs
 buildDefunSyms (DClassD _cxt name tvbs _fundeps _members) = do
   defunReifyFixity name tvbs (Just (DConT constraintName))
 buildDefunSyms _ = fail $ "Defunctionalization symbols can only be built for " ++
@@ -82,9 +83,18 @@ buildDefunSymsTypeFamilyHead default_tvb default_kind
       res_kind = default_kind (resultSigToMaybeKind result_sig)
   defunReifyFixity name arg_tvbs res_kind
 
-buildDefunSymsTySynD :: Name -> [DTyVarBndr] -> PrM [DDec]
-buildDefunSymsTySynD name tvbs =
-  defunReifyFixity name tvbs Nothing
+buildDefunSymsTySynD :: Name -> [DTyVarBndr] -> DType -> PrM [DDec]
+buildDefunSymsTySynD name tvbs rhs =
+  defunReifyFixity name tvbs mb_res_kind
+  where
+    -- In order to have a CUSK, a type synonym must annotate its right-hand
+    -- side with an explicit kind signature, so we can make use of this
+    -- property to determine the result kind when defunctionalizing the
+    -- type synonym.
+    mb_res_kind :: Maybe DKind
+    mb_res_kind = case rhs of
+                    DSigT _ k -> Just k
+                    _         -> Nothing
 
 buildDefunSymsDataD :: [DCon] -> PrM [DDec]
 buildDefunSymsDataD ctors =
@@ -173,10 +183,9 @@ defunctionalize name m_fixity m_arg_tvbs' m_res_kind' = do
                           -- Note [Defunctionalization and dependent quantification]
                           map (map_tvb_kind (substType tvb_to_type_map)) $
                           reverse m_args
-            tyfun_param = mk_tvb tyfun_name m_tyfun
             arg_names   = map extractTvbName arg_params
-            params      = arg_params ++ [tyfun_param]
-            con_eq_ct   = DConPr sameKindName `DAppPr` lhs `DAppPr` rhs
+            params      = arg_params ++ [DPlainTV tyfun_name]
+            con_eq_ct   = DConT sameKindName `DAppT` lhs `DAppT` rhs
               where
                 lhs = foldType (DConT data_name) (map DVarT arg_names) `apply` (DVarT extra_name)
                 rhs = foldType (DConT next_name) (map DVarT (arg_names ++ [extra_name]))
@@ -193,23 +202,26 @@ defunctionalize name m_fixity m_arg_tvbs' m_res_kind' = do
                                     -- If we cannot infer the return type, don't bother
                                     -- trying to construct an explicit return kind.
                       Just tyfun ->
-                        let bound_tvs = Set.fromList (map extractTvbName arg_params) `Set.union`
-                                        foldMap (foldMap fvDType) (map extractTvbKind arg_params)
-                            not_bound tvb = not (extractTvbName tvb `Set.member` bound_tvs)
+                        let bound_tvs = OSet.fromList (map extractTvbName arg_params)
+                                        `OSet.union` foldMap (foldMap fvDType)
+                                                             (map extractTvbKind arg_params)
+                            not_bound tvb = not (extractTvbName tvb `OSet.member` bound_tvs)
                             tvb_to_type tvb_name = fromMaybe (DVarT tvb_name) $
                                                    Map.lookup tvb_name tvb_to_type_map
                             -- Implements part (2)(iii) from
                             -- Note [Defunctionalization and dependent quantification]
-                            tyfun_tvbs = filter not_bound $         -- (2)(iii)(d)
-                                         toposortTyVarsOf $         -- (2)(iii)(c)
-                                         map tvb_to_type $          -- (2)(iii)(b)
-                                         Set.toList $ fvDType tyfun -- (2)(iii)(a)
+                            tyfun_tvbs = filter not_bound $     -- (2)(iii)(d)
+                                         toposortTyVarsOf $     -- (2)(iii)(c)
+                                         map tvb_to_type $      -- (2)(iii)(b)
+                                         toList $ fvDType tyfun -- (2)(iii)(a)
                         in (arg_params, Just (DForallT tyfun_tvbs [] tyfun))
             app_data_ty = foldTypeTvbs (DConT data_name) m_args
-            app_eqn     = DTySynEqn [app_data_ty, DVarT tyfun_name]
+            app_eqn     = DTySynEqn Nothing
+                                    (DConT applyName `DAppT` app_data_ty
+                                                     `DAppT` DVarT tyfun_name)
                                     (mk_rhs (m_args ++ [DPlainTV tyfun_name]))
-            app_decl    = DTySynInstD applyName app_eqn
-            suppress    = DInstanceD Nothing []
+            app_decl    = DTySynInstD app_eqn
+            suppress    = DInstanceD Nothing Nothing []
                             (DConT suppressClassName `DAppT` app_data_ty)
                             [DLetDec $ DFunD suppressMethodName
                                              [DClause []
@@ -234,10 +246,6 @@ defunctionalize name m_fixity m_arg_tvbs' m_res_kind' = do
   other_decs <- go (num_args - 1) (reverse m_arg_tvbs) m_res_kind mk_rhs
   return $ sat_dec : other_decs
   where
-    mk_tvb :: Name -> Maybe DKind -> DTyVarBndr
-    mk_tvb tvb_name Nothing  = DPlainTV tvb_name
-    mk_tvb tvb_name (Just k) = DKindedTV tvb_name k
-
     eta_expand :: [DTyVarBndr] -> Maybe DKind -> PrM ([DTyVarBndr], Maybe DKind)
     eta_expand m_arg_tvbs Nothing = pure (m_arg_tvbs, Nothing)
     eta_expand m_arg_tvbs (Just res_kind) = do
