@@ -13,7 +13,6 @@ module Data.Singletons.Single where
 import Prelude hiding ( exp )
 import Language.Haskell.TH hiding ( cxt )
 import Language.Haskell.TH.Syntax (NameSpace(..), Quasi(..))
-import Data.Singletons.Deriving.Infer
 import Data.Singletons.Deriving.Ord
 import Data.Singletons.Deriving.Bounded
 import Data.Singletons.Deriving.Enum
@@ -34,11 +33,14 @@ import Data.Singletons.Single.Eq
 import Data.Singletons.Syntax
 import Data.Singletons.Partition
 import Language.Haskell.TH.Desugar
+import qualified Language.Haskell.TH.Desugar.OMap.Strict as OMap
+import Language.Haskell.TH.Desugar.OMap.Strict (OMap)
+import qualified Language.Haskell.TH.Desugar.OSet as OSet
+import Language.Haskell.TH.Desugar.OSet (OSet)
 import qualified Data.Map.Strict as Map
 import Data.Map.Strict ( Map )
 import Data.Maybe
 import qualified Data.Set as Set
-import Data.Set ( Set )
 import Control.Monad
 import Data.List
 import qualified GHC.LanguageExtensions.Type as LangExt
@@ -128,11 +130,13 @@ singEqInstancesOnly = concatMapM singEqInstanceOnly
 singEqInstanceOnly :: DsMonad q => Name -> q [Dec]
 singEqInstanceOnly name = singEqualityInstance sEqClassDesc name
 
--- | Create instances of 'SDecide' for each type in the list.
+-- | Create instances of 'SDecide', 'TestEquality', and 'TestCoercion' for each
+-- type in the list.
 singDecideInstances :: DsMonad q => [Name] -> q [Dec]
 singDecideInstances = concatMapM singDecideInstance
 
--- | Create instance of 'SDecide' for the given type.
+-- | Create instance of 'SDecide', 'TestEquality', and 'TestCoercion' for the
+-- given type.
 singDecideInstance :: DsMonad q => Name -> q [Dec]
 singDecideInstance name = singEqualityInstance sDecideClassDesc name
 
@@ -146,9 +150,14 @@ singEqualityInstance desc@(_, _, className, _) name = do
   dcons <- concatMapM (dsCon dtvbs data_ty) cons
   let tyvars = map (DVarT . extractTvbName) dtvbs
       kind = foldType (DConT name) tyvars
-  (scons, _) <- singM [] $ mapM singCtor dcons
+  (scons, _) <- singM [] $ mapM (singCtor name) dcons
   eqInstance <- mkEqualityInstance Nothing kind dcons scons desc
-  return $ decToTH eqInstance
+  testInstances <-
+    if className == sDecideClassName
+       then traverse (mkTestInstance Nothing kind name dcons)
+                     [TestEquality, TestCoercion]
+       else pure []
+  return $ decsToTH (eqInstance:testInstances)
 
 -- | Create instances of 'SOrd' for the given types
 singOrdInstances :: DsMonad q => [Name] -> q [Dec]
@@ -178,7 +187,7 @@ singEnumInstance = singInstance mkEnumInstance "Enum"
 --
 -- (Not to be confused with 'showShowInstance'.)
 singShowInstance :: DsMonad q => Name -> q [Dec]
-singShowInstance = singInstance mkShowInstance "Show"
+singShowInstance = singInstance (mkShowInstance ForPromotion) "Show"
 
 -- | Create instances of 'SShow' for the given types
 --
@@ -198,9 +207,10 @@ showSingInstance name = do
   let tyvars    = map (DVarT . extractTvbName) dtvbs
       kind      = foldType (DConT name) tyvars
       data_decl = DataDecl name dtvbs dcons
-      deriv_show_decl = DerivedDecl { ded_mb_cxt = Nothing
-                                    , ded_type   = kind
-                                    , ded_decl   = data_decl }
+      deriv_show_decl = DerivedDecl { ded_mb_cxt     = Nothing
+                                    , ded_type       = kind
+                                    , ded_type_tycon = name
+                                    , ded_decl       = data_decl }
   (show_insts, _) <- singM [] $ singDerivedShowDecs deriv_show_decl
   pure $ decsToTH show_insts
 
@@ -210,6 +220,55 @@ showSingInstance name = do
 showSingInstances :: DsMonad q => [Name] -> q [Dec]
 showSingInstances = concatMapM showSingInstance
 
+-- | Create an instance for @'SingI' TyCon{N}@, where @N@ is the positive
+-- number provided as an argument.
+--
+-- Note that the generated code requires the use of the @QuantifiedConstraints@
+-- language extension.
+singITyConInstances :: DsMonad q => [Int] -> q [Dec]
+singITyConInstances = concatMapM singITyConInstance
+
+-- | Create an instance for @'SingI' TyCon{N}@, where @N@ is the positive
+-- number provided as an argument.
+--
+-- Note that the generated code requires the use of the @QuantifiedConstraints@
+-- language extension.
+singITyConInstance :: DsMonad q => Int -> q [Dec]
+singITyConInstance n
+  | n <= 0
+  = fail $ "Argument must be a positive number (given " ++ show n ++ ")"
+  | otherwise
+  = do as <- replicateM n (qNewName "a")
+       ks <- replicateM n (qNewName "k")
+       k_last <- qNewName "k_last"
+       f      <- qNewName "f"
+       x      <- qNewName "x"
+       let k_penult = last ks
+           k_fun = ravel (map DVarT ks) (DVarT k_last)
+           f_ty  = DVarT f
+           a_tys = map DVarT as
+           mk_fun arrow t1 t2 = arrow `DAppT` t1 `DAppT` t2
+           matchable_apply_fun   = mk_fun DArrowT                (DVarT k_penult) (DVarT k_last)
+           unmatchable_apply_fun = mk_fun (DConT tyFunArrowName) (DVarT k_penult) (DVarT k_last)
+           ctxt = [ DForallT (map DPlainTV as)
+                             (map (DAppT (DConT singIName)) a_tys)
+                             (DConT singIName `DAppT` foldType f_ty a_tys)
+                  , DConT equalityName
+                      `DAppT` (DConT applyTyConName `DSigT`
+                                mk_fun DArrowT matchable_apply_fun unmatchable_apply_fun)
+                      `DAppT` DConT applyTyConAux1Name
+                  ]
+       pure $ decToTH
+            $ DInstanceD
+                Nothing Nothing ctxt
+                (DConT singIName `DAppT` (DConT (mkTyConName n) `DAppT` (f_ty `DSigT` k_fun)))
+                [DLetDec $ DFunD singMethName
+                           [DClause [] $
+                            wrapSingFun 1 DWildCardT $
+                            DLamE [x] $
+                            DVarE withSingIName `DAppE` DVarE x
+                                                `DAppE` DVarE singMethName]]
+
 singInstance :: DsMonad q => DerivDesc q -> String -> Name -> q [Dec]
 singInstance mk_inst inst_name name = do
   (tvbs, cons) <- getDataD ("I cannot make an instance of " ++ inst_name
@@ -220,7 +279,7 @@ singInstance mk_inst inst_name name = do
   let data_decl = DataDecl name dtvbs dcons
   raw_inst <- mk_inst Nothing data_ty data_decl
   (a_inst, decs) <- promoteM [] $
-                    promoteInstanceDec Map.empty raw_inst
+                    promoteInstanceDec OMap.empty raw_inst
   decs' <- singDecsM [] $ (:[]) <$> singInstD a_inst
   return $ decsToTH (decs ++ decs')
 
@@ -297,7 +356,7 @@ buildDataLets (DataDecl _name _tvbs cons) =
 -- see comment at top of file
 buildMethLets :: UClassDecl -> [(Name, DExp)]
 buildMethLets (ClassDecl { cd_lde = LetDecEnv { lde_types = meth_sigs } }) =
-  map mk_bind (Map.toList meth_sigs)
+  map mk_bind (OMap.assocs meth_sigs)
   where
     mk_bind (meth_name, meth_ty) =
       ( meth_name
@@ -314,7 +373,7 @@ singClassD (ClassDecl { cd_cxt  = cls_cxt
                                             , lde_infix     = fixities
                                             , lde_proms     = promoted_defaults
                                             , lde_bound_kvs = meth_bound_kvs } }) =
-  bindContext [foldPredTvbs (DConPr cls_name) cls_tvbs] $ do
+  bindContext [foldTypeTvbs (DConT cls_name) cls_tvbs] $ do
     (sing_sigs, _, tyvar_names, cxts, res_kis, singIDefunss)
       <- unzip6 <$> zipWithM (singTySig no_meth_defns meth_sigs meth_bound_kvs)
                              meth_names (map promoteValRhs meth_names)
@@ -326,8 +385,8 @@ singClassD (ClassDecl { cd_cxt  = cls_cxt
     sing_meths <- mapM (uncurry (singLetDecRHS (Map.fromList tyvar_names)
                                                (Map.fromList cxts)
                                                res_ki_map))
-                       (Map.toList default_defns)
-    fixities' <- traverse (uncurry singInfixDecl) $ Map.toList fixities
+                       (OMap.assocs default_defns)
+    fixities' <- traverse (uncurry singInfixDecl) $ OMap.assocs fixities
     cls_cxt' <- mapM singPred cls_cxt
     return $ DClassD cls_cxt'
                      (singClassName cls_name)
@@ -337,15 +396,15 @@ singClassD (ClassDecl { cd_cxt  = cls_cxt
   where
     no_meth_defns = error "Internal error: can't find declared method type"
     always_sig    = error "Internal error: no signature for default method"
-    meth_names    = Map.keys meth_sigs
+    meth_names    = map fst $ OMap.assocs meth_sigs
 
     mk_default_sig meth_name (DSigD s_name sty) bound_kvs (Just res_ki) =
       DDefaultSigD s_name <$> add_constraints meth_name sty bound_kvs res_ki
     mk_default_sig _ _ _ _ = error "Internal error: a singled signature isn't a signature."
 
     add_constraints meth_name sty (_, bound_kvs) res_ki = do  -- Maybe monad
-      prom_dflt <- Map.lookup meth_name promoted_defaults
-      let default_pred = foldPred (DConPr equalityName)
+      prom_dflt <- OMap.lookup meth_name promoted_defaults
+      let default_pred = foldType (DConT equalityName)
                                 -- NB: Need the res_ki here to prevent ambiguous
                                 -- kinds in result-inferred default methods.
                                 -- See #175
@@ -382,6 +441,7 @@ singInstD (InstDecl { id_cxt = cxt, id_name = inst_name, id_arg_tys = inst_tys
     inst_kis <- mapM promoteType inst_tys
     meths <- concatMapM (uncurry sing_meth) ann_meths
     return (DInstanceD Nothing
+                       Nothing
                        cxt'
                        (foldl DAppT (DConT s_inst_name) inst_kis)
                        meths)
@@ -406,7 +466,7 @@ singInstD (InstDecl { id_cxt = cxt, id_name = inst_name, id_arg_tys = inst_tys
               -- less confusing.
               vis_cls_tvbs = drop (length cls_tvbs - length inst_kis) cls_tvbs
 
-          sing_meth_ty :: Set Name -> DType
+          sing_meth_ty :: OSet Name -> DType
                        -> SgM (DType, [Name], DCxt, DKind)
           sing_meth_ty bound_kvs inner_ty = do
             -- Make sure to expand through type synonyms here! Not doing so
@@ -416,11 +476,11 @@ singInstD (InstDecl { id_cxt = cxt, id_name = inst_name, id_arg_tys = inst_tys
               <- singType bound_kvs (promoteValRhs name) raw_ty
             pure (s_ty, tyvar_names, ctxt, res_ki)
 
-      (s_ty, tyvar_names, ctxt, m_res_ki) <- case Map.lookup name inst_sigs of
+      (s_ty, tyvar_names, ctxt, m_res_ki) <- case OMap.lookup name inst_sigs of
         Just inst_sig -> do
           -- We have an InstanceSig, so just single that type. Take care to
           -- avoid binding the variables bound by the instance head as well.
-          let inst_bound = foldMap (fvDType . predToType) cxt <> foldMap fvDType inst_kis
+          let inst_bound = foldMap fvDType (cxt ++ inst_kis)
           (s_ty, tyvar_names, ctxt, res_ki) <- sing_meth_ty inst_bound inst_sig
           pure (s_ty, tyvar_names, ctxt, Just res_ki)
         Nothing -> case mb_s_info of
@@ -435,7 +495,7 @@ singInstD (InstDecl { id_cxt = cxt, id_name = inst_name, id_arg_tys = inst_tys
 
             pure ( substType subst s_ty
                  , map extractTvbName sing_tvbs
-                 , map (substPred subst) ctxt
+                 , map (substType subst) ctxt
                  , m_res_ki )
           _ -> do
             mb_info <- dsReify name
@@ -443,8 +503,8 @@ singInstD (InstDecl { id_cxt = cxt, id_name = inst_name, id_arg_tys = inst_tys
               Just (DVarI _ (DForallT cls_tvbs _cls_pred inner_ty) _) -> do
                 let subst = mk_subst cls_tvbs
                     cls_kvb_names = foldMap (foldMap fvDType . extractTvbKind) cls_tvbs
-                    cls_tvb_names = Set.fromList $ map extractTvbName cls_tvbs
-                    cls_bound     = cls_kvb_names `Set.union` cls_tvb_names
+                    cls_tvb_names = OSet.fromList $ map extractTvbName cls_tvbs
+                    cls_bound     = cls_kvb_names `OSet.union` cls_tvb_names
                 (s_ty, tyvar_names, ctxt, res_ki) <- sing_meth_ty cls_bound inner_ty
                 pure ( substType subst s_ty
                      , tyvar_names
@@ -473,23 +533,23 @@ singLetDecEnv (LetDecEnv { lde_defns     = defns
                          , lde_proms     = proms
                          , lde_bound_kvs = bound_kvs })
               thing_inside = do
-  let prom_list = Map.toList proms
+  let prom_list = OMap.assocs proms
   (typeSigs, letBinds, tyvarNames, cxts, res_kis, singIDefunss)
     <- unzip6 <$> mapM (uncurry (singTySig defns types bound_kvs)) prom_list
-  infix_decls' <- traverse (uncurry singInfixDecl) $ Map.toList infix_decls
+  infix_decls' <- traverse (uncurry singInfixDecl) $ OMap.assocs infix_decls
   let res_ki_map = Map.fromList [ (name, res_ki) | ((name, _), Just res_ki)
                                                      <- zip prom_list res_kis ]
   bindLets letBinds $ do
     let_decs <- mapM (uncurry (singLetDecRHS (Map.fromList tyvarNames)
                                              (Map.fromList cxts)
                                              res_ki_map))
-                     (Map.toList defns)
+                     (OMap.assocs defns)
     thing <- thing_inside
     return (infix_decls' ++ typeSigs ++ let_decs, concat singIDefunss, thing)
 
-singTySig :: Map Name ALetDecRHS  -- definitions
-          -> Map Name DType       -- type signatures
-          -> Map Name (Set Name)  -- bound kind variables
+singTySig :: OMap Name ALetDecRHS  -- definitions
+          -> OMap Name DType       -- type signatures
+          -> OMap Name (OSet Name) -- bound kind variables
           -> Name -> DType   -- the type is the promoted type, not the type sig!
           -> SgM ( DLetDec               -- the new type signature
                  , (Name, DExp)          -- the let-bind entry
@@ -500,7 +560,7 @@ singTySig :: Map Name ALetDecRHS  -- definitions
                  )
 singTySig defns types bound_kvs name prom_ty =
   let sName = singValName name in
-  case Map.lookup name types of
+  case OMap.lookup name types of
     Nothing -> do
       num_args <- guess_num_args
       (sty, tyvar_names) <- mk_sing_ty num_args
@@ -528,14 +588,14 @@ singTySig defns types bound_kvs name prom_ty =
   where
     guess_num_args :: SgM Int
     guess_num_args =
-      case Map.lookup name defns of
+      case OMap.lookup name defns of
         Nothing -> fail "Internal error: promotion known for something not let-bound."
         Just (AValue _ n _) -> return n
         Just (AFunction _ n _) -> return n
 
-    lookup_bound_kvs :: SgM (Set Name)
+    lookup_bound_kvs :: SgM (OSet Name)
     lookup_bound_kvs =
-      case Map.lookup name bound_kvs of
+      case OMap.lookup name bound_kvs of
         Nothing -> fail $ "Internal error: " ++ nameBase name ++ " has no type variable "
                           ++ "bindings, despite having a type signature"
         Just kvs -> pure kvs
@@ -559,7 +619,7 @@ singLetDecRHS bound_names cxts res_kis name ld_rhs =
   bindContext (Map.findWithDefault [] name cxts) $
     case ld_rhs of
       AValue prom num_arrows exp ->
-        DValD (DVarPa (singValName name)) <$>
+        DValD (DVarP (singValName name)) <$>
         (wrapUnSingFun num_arrows prom <$> singExp exp (Map.lookup name res_kis))
       AFunction prom_fun num_arrows clauses ->
         let tyvar_names = case Map.lookup name bound_names of
@@ -603,21 +663,21 @@ singPat :: Map Name Name   -- from term-level names to type-level names
 singPat var_proms = go
   where
     go :: ADPat -> QWithAux SingDSigPaInfos SgM DPat
-    go (ADLitPa _lit) =
+    go (ADLitP _lit) =
       fail "Singling of literal patterns not yet supported"
-    go (ADVarPa name) = do
+    go (ADVarP name) = do
       tyname <- case Map.lookup name var_proms of
                   Nothing     ->
                     fail "Internal error: unknown variable when singling pattern"
                   Just tyname -> return tyname
-      pure $ DVarPa (singValName name) `DSigPa` (singFamily `DAppT` DVarT tyname)
-    go (ADConPa name pats) = DConPa (singDataConName name) <$> mapM go pats
-    go (ADTildePa pat) = do
+      pure $ DVarP (singValName name) `DSigP` (singFamily `DAppT` DVarT tyname)
+    go (ADConP name pats) = DConP (singDataConName name) <$> mapM go pats
+    go (ADTildeP pat) = do
       qReportWarning
         "Lazy pattern converted into regular pattern during singleton generation."
       go pat
-    go (ADBangPa pat) = DBangPa <$> go pat
-    go (ADSigPa prom_pat pat ty) = do
+    go (ADBangP pat) = DBangP <$> go pat
+    go (ADSigP prom_pat pat ty) = do
       pat' <- go pat
       -- Normally, calling dPatToDExp would be dangerous, since it fails if the
       -- supplied pattern contains any wildcard patterns. However, promotePat
@@ -627,7 +687,7 @@ singPat var_proms = go
       -- See Note [Singling pattern signatures].
       addElement (dPatToDExp pat', DSigT prom_pat ty)
       pure pat'
-    go ADWildPa = pure DWildPa
+    go ADWildP = pure DWildP
 
 -- | If given a non-empty list of 'SingDSigPaInfos', construct a case expression
 -- that brings singleton equality constraints into scope via pattern-matching.
@@ -638,7 +698,7 @@ mkSigPaCaseE exps_with_sigs exp
   | otherwise =
       let (exps, sigs) = unzip exps_with_sigs
           scrutinee = mkTupleDExp exps
-          pats = map (DSigPa DWildPa . DAppT (DConT singFamilyName)) sigs
+          pats = map (DSigP DWildP . DAppT (DConT singFamilyName)) sigs
       in DCaseE scrutinee [DMatch (mkTupleDPat pats) exp]
 
 -- Note [Annotate case return type]
@@ -650,6 +710,11 @@ mkSigPaCaseE exps_with_sigs exp
 -- becomes "untouchable" in the case matches. See the OutsideIn paper. But,
 -- during singletonization, we *know* the return type. So, just add a type
 -- annotation. See #54.
+--
+-- In particular, we add a type annotation in a somewhat unorthodox fashion.
+-- Instead of the usual `(x :: t)`, we use `id @t x`. See
+-- Note [The id hack; or, how singletons learned to stop worrying and avoid
+-- kind generalization] for an explanation of why we do this.
 
 -- Note [Why error is so special]
 -- ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
@@ -765,14 +830,17 @@ singExp (ADLamE ty_names prom_lam names exp) _res_ki = do
   -- So: build a case
   let caseExp = DCaseE (mkTupleDExp (map DVarE sNames))
                        [DMatch (mkTupleDPat
-                                (map ((DWildPa `DSigPa`) .
+                                (map ((DWildP `DSigP`) .
                                       (singFamily `DAppT`) .
                                       DVarT) ty_names)) exp']
   return $ wrapSingFun (length names) prom_lam $ DLamE sNames caseExp
 singExp (ADCaseE exp matches ret_ty) res_ki =
-    -- See Note [Annotate case return type]
-  DSigE <$> (DCaseE <$> singExp exp Nothing <*> mapM (singMatch res_ki) matches)
-        <*> pure (singFamily `DAppT` (ret_ty `maybeSigT` res_ki))
+    -- See Note [Annotate case return type] and
+    --     Note [The id hack; or, how singletons learned to stop worrying and
+    --           avoid kind generalization]
+  DAppE (DAppTypeE (DVarE 'id)
+                   (singFamily `DAppT` (ret_ty `maybeSigT` res_ki)))
+    <$> (DCaseE <$> singExp exp Nothing <*> mapM (singMatch res_ki) matches)
 singExp (ADLetE env exp) res_ki = do
   -- We intentionally discard the SingI instances for exp's defunctionalization
   -- symbols, as we also do not generate the declarations for the
@@ -783,12 +851,13 @@ singExp (ADSigE prom_exp exp ty) _ = do
   exp' <- singExp exp (Just ty)
   pure $ DSigE exp' $ DConT singFamilyName `DAppT` DSigT prom_exp ty
 
--- See Note [DerivedDecl]
+-- See Note [DerivedDecl] in Data.Singletons.Syntax
 singDerivedEqDecs :: DerivedEqDecl -> SgM [DDec]
-singDerivedEqDecs (DerivedDecl { ded_mb_cxt = mb_ctxt
-                               , ded_type   = ty
-                               , ded_decl   = DataDecl _ _ cons }) = do
-  (scons, _) <- singM [] $ mapM singCtor cons
+singDerivedEqDecs (DerivedDecl { ded_mb_cxt     = mb_ctxt
+                               , ded_type       = ty
+                               , ded_type_tycon = ty_tycon
+                               , ded_decl       = DataDecl _ _ cons }) = do
+  (scons, _) <- singM [] $ mapM (singCtor ty_tycon) cons
   mb_sctxt <- mapM (mapM singPred) mb_ctxt
   kind <- promoteType ty
   sEqInst <- mkEqualityInstance mb_sctxt kind cons scons sEqClassDesc
@@ -801,35 +870,49 @@ singDerivedEqDecs (DerivedDecl { ded_mb_cxt = mb_ctxt
   -- all occurrences of SEq with SDecide in the context.
   let mb_sctxtDecide = fmap (map sEqToSDecide) mb_sctxt
   sDecideInst <- mkEqualityInstance mb_sctxtDecide kind cons scons sDecideClassDesc
-  return [sEqInst, sDecideInst]
+  testInsts <- traverse (mkTestInstance mb_sctxtDecide kind ty_tycon cons)
+                        [TestEquality, TestCoercion]
+  return (sEqInst:sDecideInst:testInsts)
 
 -- Walk a DPred, replacing all occurrences of SEq with SDecide.
 sEqToSDecide :: DPred -> DPred
-sEqToSDecide = modifyConNameDPred $ \n ->
+sEqToSDecide = modifyConNameDType $ \n ->
   -- Why don't we directly compare n to sEqClassName? Because n is almost certainly
   -- produced from a call to singClassName, which uses unqualified Names. Ugh.
   if nameBase n == nameBase sEqClassName
      then sDecideClassName
      else n
 
--- See Note [DerivedDecl]
+-- See Note [DerivedDecl] in Data.Singletons.Syntax
 singDerivedShowDecs :: DerivedShowDecl -> SgM [DDec]
-singDerivedShowDecs (DerivedDecl { ded_mb_cxt = mb_cxt
-                                 , ded_type   = ty
-                                 , ded_decl   = DataDecl _ _ cons }) = do
-    z <- qNewName "z"
-    -- Derive the Show instance for the singleton type, like this:
+singDerivedShowDecs (DerivedDecl { ded_mb_cxt     = mb_cxt
+                                 , ded_type       = ty
+                                 , ded_type_tycon = ty_tycon
+                                 , ded_decl       = data_decl }) = do
+    -- Generate a Show instance for a singleton type, like this:
     --
-    --   deriving instance (ShowSing a, ShowSing b) => Sing (Sing (z :: Either a b))
+    --   instance (ShowSing a, ShowSing b) => Show (SEither (z :: Either a b)) where
+    --     showsPrec p (SLeft (sl :: Sing l)) =
+    --       showParen (p > 10) $ showString "SLeft " . showsPrec 11 sl
+    --         :: ShowSing' l => ShowS
+    --     showsPrec p (SRight (sr :: Sing r)) =
+    --       showParen (p > 10) $ showString "SRight " . showsPrec 11 sr
+    --         :: ShowSing' r => ShowS
     --
     -- Be careful: we want to generate an instance context that uses ShowSing,
     -- not SShow.
-    show_cxt <- inferConstraintsDef (fmap mkShowSingContext mb_cxt)
-                                    (DConPr showSingName)
-                                    ty cons
-    let show_inst = DStandaloneDerivD Nothing show_cxt
-                      (DConT showName `DAppT` (singFamily `DAppT` DSigT (DVarT z) ty))
-    pure [show_inst]
+    show_sing_inst <- mkShowInstance (ForShowSing ty_tycon) mb_cxt ty data_decl
+    pure [toInstanceD show_sing_inst]
+  where
+    toInstanceD :: UInstDecl -> DDec
+    toInstanceD (InstDecl { id_cxt = cxt, id_name = inst_name
+                          , id_arg_tys = inst_tys, id_meths = ann_meths }) =
+      DInstanceD Nothing Nothing cxt (foldType (DConT inst_name) inst_tys)
+                 (map (DLetDec . toFunD) ann_meths)
+
+    toFunD :: (Name, ULetDecRHS) -> DLetDec
+    toFunD (fun_name, UFunction clauses) = DFunD fun_name clauses
+    toFunD (val_name, UValue rhs)        = DValD (DVarP val_name) rhs
 
 isException :: DExp -> Bool
 isException (DVarE n)             = nameBase n == "sUndefined"
@@ -872,3 +955,85 @@ singLit lit =
 maybeSigT :: DType -> Maybe DKind -> DType
 maybeSigT ty Nothing   = ty
 maybeSigT ty (Just ki) = ty `DSigT` ki
+
+{-
+Note [The id hack; or, how singletons learned to stop worrying and avoid kind generalization]
+~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+GHC 8.8 was a time of great change. In particular, 8.8 debuted a fix for
+Trac #15141 (decideKindGeneralisationPlan is too complicated). To fix this, a
+wily GHC developer—who shall remain unnamed, but whose username rhymes with
+schmoldfire—decided to make decideKindGeneralisationPlan less complicated by,
+well, removing the whole thing. One consequence of this is that local
+definitions are now kind-generalized (whereas they would not have been
+previously).
+
+While schmoldfire had the noblest of intentions when authoring his fix, he
+unintentionally made life much harder for singletons. Why? Consider the
+following program:
+
+  class Foo a where
+    bar :: a -> (a -> b) -> b
+    baz :: a
+
+  quux :: Foo a => a -> a
+  quux x = x `bar` \_ -> baz
+
+When singled, this program will turn into something like this:
+
+  type family Quux (x :: a) :: a where
+    Quux x = Bar x (LambdaSym1 x)
+
+  sQuux :: forall a (x :: a). SFoo a => Sing x -> Sing (Quux x :: a)
+  sQuux (sX :: Sing x)
+    = sBar sX
+        ((singFun1 @(LambdaSym1 x))
+           (\ sArg
+              -> case sArg of {
+                   (_ :: Sing arg)
+                     -> (case sArg of { _ -> sBaz }) ::
+                          Sing (Case x arg arg) }))
+
+  type family Case x arg t where
+    Case x arg _ = Baz
+  type family Lambda x t where
+    Lambda x arg = Case x arg arg
+  data LambdaSym1 x t
+  type instance Apply (LambdaSym1 x) t = Lambda x t
+
+The high-level bit is the explicit `Sing (Case x arg arg)` signature. Question:
+what is the kind of `Case x arg arg`? The answer depends on whether local
+definitions are kind-generalized or not!
+
+1. If local definitions are *not* kind-generalized (i.e., the status quo before
+   GHC 8.8), then `Case x arg arg :: a`.
+2. If local definitions *are* kind-generalized (i.e., the status quo in GHC 8.8
+   and later), then `Case x arg arg :: k` for some fresh kind variable `k`.
+
+Unfortunately, the kind of `Case x arg arg` *must* be `a` in order for `sQuux`
+to type-check. This means that the code above suddenly stopped working in GHC
+8.8. What's more, we can't just remove these explicit signatures, as there is
+code elsewhere in `singletons` that crucially relies on them to guide type
+inference along (e.g., `sShowParen` in `Data.Singletons.Prelude.Show`).
+
+Luckily, there is an ingenious hack that lets us the benefits of explicit
+signatures without the pain of kind generalization: our old friend, the `id`
+function. The plan is as follows: instead of generating this code:
+
+  (case sArg of ...) :: Sing (Case x arg arg)
+
+We instead generate this code:
+
+  id @(Sing (Case x arg arg)) (case sArg of ...)
+
+That's it! This works because visible type arguments in terms do not get kind-
+generalized, unlike top-level or local signatures. Now `Case x arg arg`'s kind
+is not generalized, and all is well. We dub this: the `id` hack.
+
+One might wonder: will we need the `id` hack around forever? Perhaps not. While
+GHC 8.8 removed the decideKindGeneralisationPlan function, there have been
+rumblings that a future version of GHC may bring it back (in a limited form).
+If this happens, it is possibly that GHC's attitude towards kind-generalizing
+local definitons may change *again*, which could conceivably render the `id`
+hack unnecessary. This is all speculation, of course, so all we can do now is
+wait and revisit this design at a later date.
+-}
