@@ -11,6 +11,7 @@ Singletonizes constructors.
 module Data.Singletons.Single.Data where
 
 import Language.Haskell.TH.Desugar
+import Language.Haskell.TH.Desugar.OSet (OSet)
 import Language.Haskell.TH.Syntax
 import Data.Singletons.Single.Defun
 import Data.Singletons.Single.Monad
@@ -21,8 +22,7 @@ import Data.Singletons.Util
 import Data.Singletons.Names
 import Data.Singletons.Syntax
 import Control.Monad
-import qualified Data.Set as Set
-import Data.Set (Set)
+import Data.Foldable
 
 -- We wish to consider the promotion of "Rep" to be *
 -- not a promoted data constructor.
@@ -30,7 +30,7 @@ singDataD :: DataDecl -> SgM [DDec]
 singDataD (DataDecl name tvbs ctors) = do
   let tvbNames = map extractTvbName tvbs
   k <- promoteType (foldType (DConT name) (map DVarT tvbNames))
-  ctors' <- mapM singCtor ctors
+  ctors' <- mapM (singCtor name) ctors
   ctorFixities <-
     -- try to reify the fixity declarations for the constructors and then
     -- singletonize them. In case the reification fails, we default to an
@@ -48,11 +48,11 @@ singDataD (DataDecl name tvbs ctors) = do
   toSingClauses       <- mapM mkToSingClause ctors
   emptyToSingClause   <- mkEmptyToSingClause
   let singKindInst =
-        DInstanceD Nothing
+        DInstanceD Nothing Nothing
                    (map (singKindConstraint . DVarT) tvbNames)
                    (DAppT (DConT singKindClassName) k)
-                   [ DTySynInstD demoteName $ DTySynEqn
-                      [k]
+                   [ DTySynInstD $ DTySynEqn Nothing
+                      (DConT demoteName `DAppT` k)
                       (foldType (DConT name)
                         (map (DAppT demote . DVarT) tvbNames))
                    , DLetDec $ DFunD fromSingName
@@ -60,15 +60,17 @@ singDataD (DataDecl name tvbs ctors) = do
                    , DLetDec $ DFunD toSingName
                                (toSingClauses   `orIfEmpty` [emptyToSingClause]) ]
 
-  -- e.g. type SNat = (Sing :: Nat -> Type)
-  let kindedSingTy = DArrowT `DAppT` k `DAppT` DConT typeKindName
-      kindedSynInst =
-        DTySynD (singTyConName name)
-                []
-                (singFamily `DSigT` kindedSingTy)
+  let singDataName = singTyConName name
+      -- e.g. type instance Sing @Nat = SNat
+      singSynInst =
+        DTySynInstD $ DTySynEqn Nothing
+                                (DConT singFamilyName `DAppKindT` k)
+                                (DConT singDataName)
+      kindedSingTy = DForallT (map DPlainTV tvbNames) [] $
+                     DArrowT `DAppT` k `DAppT` DConT typeKindName
 
-  return $ (DDataInstD Data [] singFamilyName [] (Just kindedSingTy) ctors' []) :
-           kindedSynInst :
+  return $ (DDataD Data [] singDataName [] (Just kindedSingTy) ctors' []) :
+           singSynInst :
            singKindInst :
            ctorFixities
   where -- in the Rep case, the names of the constructors are in the wrong scope
@@ -83,7 +85,7 @@ singDataD (DataDecl name tvbs ctors) = do
           let (cname, numArgs) = extractNameArgs c
           cname' <- mkConName cname
           varNames <- replicateM numArgs (qNewName "b")
-          return $ DClause [DConPa (singDataConName cname) (map DVarPa varNames)]
+          return $ DClause [DConP (singDataConName cname) (map DVarP varNames)]
                            (foldExp
                               (DConE cname')
                               (map (DAppE (DVarE fromSingName) . DVarE) varNames))
@@ -98,9 +100,9 @@ singDataD (DataDecl name tvbs ctors) = do
           let varPats        = zipWith mkToSingVarPat varNames promoted
               recursiveCalls = zipWith mkRecursiveCall varNames promoted
           return $
-            DClause [DConPa cname' varPats]
+            DClause [DConP cname' varPats]
                     (multiCase recursiveCalls
-                               (map (DConPa someSingDataName . listify . DVarPa)
+                               (map (DConP someSingDataName . listify . DVarP)
                                     svarNames)
                                (DAppE (DConE someSingDataName)
                                          (foldExp (DConE (singDataConName cname))
@@ -108,7 +110,7 @@ singDataD (DataDecl name tvbs ctors) = do
 
         mkToSingVarPat :: Name -> DKind -> DPat
         mkToSingVarPat varName ki =
-          DSigPa (DVarPa varName) (DAppT (DConT demoteName) ki)
+          DSigP (DVarP varName) (DAppT (DConT demoteName) ki)
 
         mkRecursiveCall :: Name -> DKind -> DExp
         mkRecursiveCall var_name ki =
@@ -118,21 +120,21 @@ singDataD (DataDecl name tvbs ctors) = do
         mkEmptyFromSingClause :: SgM DClause
         mkEmptyFromSingClause = do
           x <- qNewName "x"
-          pure $ DClause [DVarPa x]
+          pure $ DClause [DVarP x]
                $ DCaseE (DVarE x) []
 
         mkEmptyToSingClause :: SgM DClause
         mkEmptyToSingClause = do
           x <- qNewName "x"
-          pure $ DClause [DVarPa x]
+          pure $ DClause [DVarP x]
                $ DConE someSingDataName `DAppE` DCaseE (DVarE x) []
 
 -- refine a constructor.
-singCtor :: DCon -> SgM DCon
+singCtor :: Name -> DCon -> SgM DCon
  -- polymorphic constructors are handled just
  -- like monomorphic ones -- the polymorphism in
  -- the kind is automatic
-singCtor (DCon _tvbs cxt name fields rty)
+singCtor dataName (DCon _tvbs cxt name fields rty)
   | not (null cxt)
   = fail "Singling of constrained constructors not yet supported"
   | otherwise
@@ -147,16 +149,16 @@ singCtor (DCon _tvbs cxt name fields rty)
   let bound_kvs = foldMap fvDType kinds
   args <- zipWithM (buildArgType bound_kvs) types indices
   rty' <- promoteType rty
-  let tvbs = map DPlainTV (Set.toList bound_kvs) ++ zipWith DKindedTV indexNames kinds
+  let tvbs = map DPlainTV (toList bound_kvs) ++ zipWith DKindedTV indexNames kinds
       kindedIndices = zipWith DSigT indices kinds
 
   -- SingI instance for data constructor
   emitDecs
-    [DInstanceD Nothing
-                (map (DAppPr (DConPr singIName)) indices)
+    [DInstanceD Nothing Nothing
+                (map (DAppT (DConT singIName)) indices)
                 (DAppT (DConT singIName)
                        (foldType pCon kindedIndices))
-                [DLetDec $ DValD (DVarPa singMethName)
+                [DLetDec $ DValD (DVarP singMethName)
                        (foldExp sCon (map (const $ DVarE singMethName) types))]]
   -- SingI instances for defunctionalization symbols. Note that we don't
   -- support contexts in constructors at the moment, so it's fine for now to
@@ -174,8 +176,8 @@ singCtor (DCon _tvbs cxt name fields rty)
                 []
                 sName
                 conFields
-                (DConT singFamilyName `DAppT` foldType pCon indices)
-  where buildArgType :: Set Name -> DType -> DType -> SgM DType
+                (DConT (singTyConName dataName) `DAppT` foldType pCon indices)
+  where buildArgType :: OSet Name -> DType -> DType -> SgM DType
         buildArgType bound_kvs ty index = do
           (ty', _, _, _, _, _) <- singType bound_kvs index ty
           return ty'
