@@ -9,16 +9,17 @@ Singletonizes types.
 module Data.Singletons.Single.Type where
 
 import Language.Haskell.TH.Desugar
+import qualified Language.Haskell.TH.Desugar.OSet as OSet
+import Language.Haskell.TH.Desugar.OSet (OSet)
 import Language.Haskell.TH.Syntax
 import Data.Singletons.Names
 import Data.Singletons.Single.Monad
 import Data.Singletons.Promote.Type
 import Data.Singletons.Util
 import Control.Monad
-import qualified Data.Set as Set
-import Data.Set (Set)
+import Data.Foldable
 
-singType :: Set Name       -- the set of bound kind variables in this scope
+singType :: OSet Name      -- the set of bound kind variables in this scope
                            -- see Note [Explicitly binding kind variables]
                            -- in Data.Singletons.Promote.Monad
          -> DType          -- the promoted version of the thing classified by...
@@ -41,9 +42,9 @@ singType bound_kvs prom ty = do
       tau   = ravel args' res'
       -- Make sure to subtract out the bound variables currently in scope, lest we
       -- accidentally shadow them in this type signature.
-      kv_names_to_bind = foldMap fvDType (prom_args ++ map predToType cxt' ++ [prom_res])
-                             Set.\\ bound_kvs
-      kvs_to_bind      = Set.toList kv_names_to_bind
+      kv_names_to_bind = foldMap fvDType (prom_args ++ cxt' ++ [prom_res])
+                            OSet.\\ bound_kvs
+      kvs_to_bind      = toList kv_names_to_bind
   let ty' = DForallT (map DPlainTV kvs_to_bind ++ zipWith DKindedTV arg_names prom_args)
                      cxt' tau
   return (ty', num_args, arg_names, cxt, prom_args, prom_res)
@@ -51,19 +52,24 @@ singType bound_kvs prom ty = do
 singPred :: DPred -> SgM DPred
 singPred = singPredRec []
 
-singPredRec :: [DType] -> DPred -> SgM DPred
-singPredRec _cxt (DForallPr {}) =
+singPredRec :: [DTypeArg] -> DPred -> SgM DPred
+singPredRec _cxt (DForallT {}) =
   fail "Singling of quantified constraints not yet supported"
-singPredRec ctx (DAppPr pr ty) = singPredRec (ty : ctx) pr
-singPredRec _ctx (DSigPr _pr _ki) =
+singPredRec ctx (DAppT pr ty) = singPredRec (DTANormal ty : ctx) pr
+singPredRec ctx (DAppKindT pr ki) = singPredRec (DTyArg ki : ctx) pr
+singPredRec _ctx (DSigT _pr _ki) =
   fail "Singling of constraints with explicit kinds not yet supported"
-singPredRec _ctx (DVarPr _n) =
+singPredRec _ctx (DVarT _n) =
   fail "Singling of contraint variables not yet supported"
-singPredRec ctx (DConPr n)
+singPredRec ctx (DConT n)
   | n == equalityName
   = fail "Singling of type equality constraints not yet supported"
   | otherwise = do
-    kis <- mapM promoteType ctx
+    kis <- mapM promoteTypeArg ctx
     let sName = singClassName n
-    return $ foldPred (DConPr sName) kis
-singPredRec _ctx DWildCardPr = return DWildCardPr  -- it just might work
+    return $ applyDType (DConT sName) kis
+singPredRec _ctx DWildCardT = return DWildCardT  -- it just might work
+singPredRec _ctx DArrowT =
+  fail "(->) spotted at head of a constraint"
+singPredRec _ctx (DLitT {}) =
+  fail "Type-level literal spotted at head of a constraint"
