@@ -8,7 +8,7 @@ and contains various other AST definitions.
 -}
 
 {-# LANGUAGE DataKinds, TypeFamilies, PolyKinds, DeriveDataTypeable,
-             StandaloneDeriving, FlexibleInstances, ConstraintKinds #-}
+             FlexibleInstances, ConstraintKinds #-}
 
 module Data.Singletons.Syntax where
 
@@ -16,10 +16,9 @@ import Prelude hiding ( exp )
 import Data.Kind (Constraint, Type)
 import Language.Haskell.TH.Syntax hiding (Type)
 import Language.Haskell.TH.Desugar
-import Data.Map.Strict ( Map )
-import qualified Data.Map.Strict as Map
-import Data.Set ( Set )
-import Data.Semigroup (Semigroup(..))
+import qualified Language.Haskell.TH.Desugar.OMap.Strict as OMap
+import Language.Haskell.TH.Desugar.OMap.Strict (OMap)
+import Language.Haskell.TH.Desugar.OSet (OSet)
 
 type VarPromotions = [(Name, Name)] -- from term-level name to type-level name
 
@@ -27,7 +26,7 @@ type VarPromotions = [(Name, Name)] -- from term-level name to type-level name
 data PromDPatInfos = PromDPatInfos
   { prom_dpat_vars    :: VarPromotions
       -- Maps term-level pattern variables to their promoted, type-level counterparts.
-  , prom_dpat_sig_kvs :: Set Name
+  , prom_dpat_sig_kvs :: OSet Name
       -- Kind variables bound by DSigPas.
       -- See Note [Explicitly binding kind variables] in Data.Singletons.Promote.Monad
   }
@@ -49,7 +48,7 @@ type SingDSigPaInfos = [(DExp, DType)]
 data DataDecl = DataDecl Name [DTyVarBndr] [DCon]
 
 -- The parts of type synonyms that are relevant to singletons.
-data TySynDecl = TySynDecl Name [DTyVarBndr]
+data TySynDecl = TySynDecl Name [DTyVarBndr] DType
 
 -- The parts of open type families that are relevant to singletons.
 type OpenTypeFamilyDecl = TypeFamilyDecl 'Open
@@ -73,7 +72,7 @@ data ClassDecl ann = ClassDecl { cd_cxt  :: DCxt
 data InstDecl  ann = InstDecl { id_cxt     :: DCxt
                               , id_name    :: Name
                               , id_arg_tys :: [DType]
-                              , id_sigs    :: Map Name DType
+                              , id_sigs    :: OMap Name DType
                               , id_meths   :: [(Name, LetDecRHS ann)] }
 
 type UClassDecl = ClassDecl Unannotated
@@ -106,15 +105,15 @@ data ADExp = ADVarE Name
                     ADExp DType
 
 -- A DPat with a pattern-signature node annotated with its type-level equivalent
-data ADPat = ADLitPa Lit
-           | ADVarPa Name
-           | ADConPa Name [ADPat]
-           | ADTildePa ADPat
-           | ADBangPa ADPat
-           | ADSigPa DType -- The promoted pattern. Will not contain any wildcards,
-                           -- as per Note [Singling pattern signatures]
-                     ADPat DType
-           | ADWildPa
+data ADPat = ADLitP Lit
+           | ADVarP Name
+           | ADConP Name [ADPat]
+           | ADTildeP ADPat
+           | ADBangP ADPat
+           | ADSigP DType -- The promoted pattern. Will not contain any wildcards,
+                          -- as per Note [Singling pattern signatures]
+                    ADPat DType
+           | ADWildP
 
 data ADMatch = ADMatch VarPromotions ADPat ADExp
 data ADClause = ADClause VarPromotions
@@ -145,11 +144,11 @@ type ALetDecRHS = LetDecRHS Annotated
 type ULetDecRHS = LetDecRHS Unannotated
 
 data LetDecEnv ann = LetDecEnv
-                   { lde_defns :: Map Name (LetDecRHS ann)
-                   , lde_types :: Map Name DType   -- type signatures
-                   , lde_infix :: Map Name Fixity  -- infix declarations
-                   , lde_proms :: IfAnn ann (Map Name DType) () -- possibly, promotions
-                   , lde_bound_kvs :: IfAnn ann (Map Name (Set Name)) ()
+                   { lde_defns :: OMap Name (LetDecRHS ann)
+                   , lde_types :: OMap Name DType  -- type signatures
+                   , lde_infix :: OMap Name Fixity -- infix declarations
+                   , lde_proms :: IfAnn ann (OMap Name DType) () -- possibly, promotions
+                   , lde_bound_kvs :: IfAnn ann (OMap Name (OSet Name)) ()
                      -- The set of bound variables in scope.
                      -- See Note [Explicitly binding kind variables]
                      -- in Data.Singletons.Promote.Monad
@@ -162,16 +161,16 @@ instance Semigroup ULetDecEnv where
     LetDecEnv (defns1 <> defns2) (types1 <> types2) (infx1 <> infx2) () ()
 
 instance Monoid ULetDecEnv where
-  mempty = LetDecEnv Map.empty Map.empty Map.empty () ()
+  mempty = LetDecEnv OMap.empty OMap.empty OMap.empty () ()
 
 valueBinding :: Name -> ULetDecRHS -> ULetDecEnv
-valueBinding n v = emptyLetDecEnv { lde_defns = Map.singleton n v }
+valueBinding n v = emptyLetDecEnv { lde_defns = OMap.singleton n v }
 
 typeBinding :: Name -> DType -> ULetDecEnv
-typeBinding n t = emptyLetDecEnv { lde_types = Map.singleton n t }
+typeBinding n t = emptyLetDecEnv { lde_types = OMap.singleton n t }
 
 infixDecl :: Fixity -> Name -> ULetDecEnv
-infixDecl f n = emptyLetDecEnv { lde_infix = Map.singleton n f }
+infixDecl f n = emptyLetDecEnv { lde_infix = OMap.singleton n f }
 
 emptyLetDecEnv :: ULetDecEnv
 emptyLetDecEnv = mempty
@@ -182,7 +181,7 @@ buildLetDecEnv = go emptyLetDecEnv
     go acc [] = return acc
     go acc (DFunD name clauses : rest) =
       go (valueBinding name (UFunction clauses) <> acc) rest
-    go acc (DValD (DVarPa name) exp : rest) =
+    go acc (DValD (DVarP name) exp : rest) =
       go (valueBinding name (UValue exp) <> acc) rest
     go acc (dec@(DValD {}) : rest) = do
       flattened <- flattenDValD dec
@@ -195,9 +194,10 @@ buildLetDecEnv = go emptyLetDecEnv
 
 -- See Note [DerivedDecl]
 data DerivedDecl (cls :: Type -> Constraint) = DerivedDecl
-  { ded_mb_cxt :: Maybe DCxt
-  , ded_type   :: DType
-  , ded_decl   :: DataDecl
+  { ded_mb_cxt     :: Maybe DCxt
+  , ded_type       :: DType
+  , ded_type_tycon :: Name
+  , ded_decl       :: DataDecl
   }
 
 type DerivedEqDecl   = DerivedDecl Eq
@@ -219,15 +219,16 @@ information to recreate the derived instance:
    a deriving clause (ded_mb_cxt)
 2. The datatype, applied to some number of type arguments, as in the
    instance declaration (ded_type)
-3. The datatype's original information, as provided through DataDecl (ded_decl)
+3. The datatype name (ded_type_tycon), cached for convenience
+4. The datatype's constructors (ded_cons)
 
 Why are these instances handled outside of partitionDecs?
 
 * Deriving Eq in singletons not only derives PEq/SEq instances, but it also
-  derives SDecide instances. This additional complication makes Eq difficult
-  to integrate with the other deriving machinery, so we handle it specially
-  in Data.Singletons.Promote and Data.Singletons.Single (depending on the task
-  at hand).
+  derives SDecide, TestEquality, and TestCoercion instances. This additional
+  complication makes Eq difficult to integrate with the other deriving
+  machinery, so we handle it specially in Data.Singletons.Promote and
+  Data.Singletons.Single (depending on the task at hand).
 * Deriving Show in singletons not only derives PShow/SShow instances, but it
   also derives Show instances for singletons types. To make this work,
   we let partitionDecs handle the PShow/SShow instances, but we also stick the
