@@ -19,8 +19,7 @@
 
 module Data.Singletons.TypeLits (
   Nat, Symbol,
-  Sing(SNat, SSym),
-  SNat, SSymbol, withKnownNat, withKnownSymbol,
+  Sing, SNat(..), SSymbol(..), withKnownNat, withKnownSymbol,
   Error, sError,
   ErrorWithoutStackTrace, sErrorWithoutStackTrace,
   Undefined, sUndefined,
