@@ -9,8 +9,7 @@ Users of the package should not need to consult this file.
 
 {-# LANGUAGE TypeSynonymInstances, FlexibleInstances, RankNTypes,
              TemplateHaskell, GeneralizedNewtypeDeriving,
-             MultiParamTypeClasses, StandaloneDeriving,
-             UndecidableInstances, MagicHash, UnboxedTuples,
+             MultiParamTypeClasses, UndecidableInstances, MagicHash,
              LambdaCase, NoMonomorphismRestriction #-}
 
 module Data.Singletons.Util where
@@ -33,7 +32,6 @@ import Data.Traversable
 import Data.Generics
 import Data.Maybe
 import Data.Void
-import Control.Monad.Fail ( MonadFail )
 
 -- The list of types that singletons processes by default
 basicTypes :: [Name]
@@ -92,7 +90,7 @@ qReportError :: Quasi q => String -> q ()
 qReportError = qReport True
 
 -- | Generate a new Unique
-qNewUnique :: DsMonad q => q Int
+qNewUnique :: DsMonad q => q Uniq
 qNewUnique = do
   Name _ flav <- qNewName "x"
   case flav of
@@ -203,7 +201,7 @@ suffixName ident symb n =
 -- convert a number into both alphanumeric and symoblic forms
 uniquePrefixes :: String   -- alphanumeric prefix
                -> String   -- symbolic prefix
-               -> Int
+               -> Uniq
                -> (String, String)  -- (alphanum, symbolic)
 uniquePrefixes alpha symb n = (alpha ++ n_str, symb ++ convert n_str)
   where
@@ -253,15 +251,6 @@ ravel :: [DType] -> DType -> DType
 ravel []    res  = res
 ravel (h:t) res = DAppT (DAppT DArrowT h) (ravel t res)
 
--- | Convert a 'DPred' to a 'DType'.
-predToType :: DPred -> DType
-predToType (DForallPr tvbs cxt p) = DForallT tvbs cxt (predToType p)
-predToType (DAppPr p t)           = DAppT (predToType p) t
-predToType (DSigPr p k)           = DSigT (predToType p) k
-predToType (DVarPr n)             = DVarT n
-predToType (DConPr n)             = DConT n
-predToType DWildCardPr            = DWildCardT
-
 -- count the number of arguments in a type
 countArgs :: DType -> Int
 countArgs ty = length args
@@ -298,9 +287,10 @@ substType subst (DForallT tvbs cxt inner_ty)
   = DForallT tvbs' cxt' inner_ty'
   where
     (subst', tvbs') = mapAccumL subst_tvb subst tvbs
-    cxt'            = map (substPred subst') cxt
+    cxt'            = map (substType subst') cxt
     inner_ty'       = substType subst' inner_ty
 substType subst (DAppT ty1 ty2) = substType subst ty1 `DAppT` substType subst ty2
+substType subst (DAppKindT ty ki) = substType subst ty `DAppKindT` substType subst ki
 substType subst (DSigT ty ki) = substType subst ty `DSigT` substType subst ki
 substType subst (DVarT n) =
   case Map.lookup n subst of
@@ -311,22 +301,6 @@ substType _ ty@(DArrowT)  = ty
 substType _ ty@(DLitT {}) = ty
 substType _ ty@DWildCardT = ty
 
-substPred :: Map Name DType -> DPred -> DPred
-substPred subst pred | Map.null subst = pred
-substPred subst (DForallPr tvbs cxt inner_pred)
-  = DForallPr tvbs' cxt' inner_pred'
-  where
-    (subst', tvbs') = mapAccumL subst_tvb subst tvbs
-    cxt'            = map (substPred subst') cxt
-    inner_pred'     = substPred subst' inner_pred
-substPred subst (DAppPr pred ty) =
-  DAppPr (substPred subst pred) (substType subst ty)
-substPred subst (DSigPr pred ki) =
-  DSigPr (substPred subst pred) (substKind subst ki)
-substPred _ pred@(DVarPr {}) = pred
-substPred _ pred@(DConPr {}) = pred
-substPred _ pred@DWildCardPr = pred
-
 subst_tvb :: Map Name DKind -> DTyVarBndr -> (Map Name DKind, DTyVarBndr)
 subst_tvb s tvb@(DPlainTV n) = (Map.delete n s, tvb)
 subst_tvb s (DKindedTV n k)  = (Map.delete n s, DKindedTV n (substKind s k))
@@ -343,34 +317,6 @@ foldType = foldl DAppT
 foldTypeTvbs :: DType -> [DTyVarBndr] -> DType
 foldTypeTvbs ty = foldType ty . map tvbToType
 
--- apply a pred to a list of types
-foldPred :: DPred -> [DType] -> DPred
-foldPred = foldl DAppPr
-
--- apply a pred to a list of type variable binders
-foldPredTvbs :: DPred -> [DTyVarBndr] -> DPred
-foldPredTvbs pr = foldPred pr . map tvbToType
-
--- | Decompose an applied type into its individual components. For example, this:
---
--- @
--- Either Int Char
--- @
---
--- would be unfolded to this:
---
--- @
--- Either :| [Int, Char]
--- @
-unfoldType :: DType -> NonEmpty DType
-unfoldType = go []
-  where
-    go :: [DType] -> DType -> NonEmpty DType
-    go acc (DAppT t1 t2)    = go (t2:acc) t1
-    go acc (DSigT t _)      = go acc t
-    go acc (DForallT _ _ t) = go acc t
-    go acc t                = t :| acc
-
 -- Construct a data type's variable binders, possibly using fresh variables
 -- from the data type's kind signature.
 buildDataDTvbs :: DsMonad q => [DTyVarBndr] -> Maybe DKind -> q [DTyVarBndr]
