@@ -1,12 +1,10 @@
-{-# OPTIONS_GHC -Wno-unused-imports #-}
-
 module Singletons.Star where
 
 import Data.Singletons.Prelude
 import Data.Singletons.Decide
 import Data.Singletons.CustomStar
 import Singletons.Nat
-import Data.Kind
+import Data.Kind (Type)
 
 data Vec :: Type -> Nat -> Type where
   VNil :: Vec a Zero
