@@ -1,7 +1,7 @@
 module Singletons.T145 where
 
 import Data.Singletons.TH
-import Data.Kind
+import Data.Kind (Type)
 
 $(singletons [d|
   class Column (f :: Type -> Type) where
